@@ -88,6 +88,27 @@ static uint64_t last_relay_viewer_session_id;
 static pthread_mutex_t last_relay_viewer_session_id_lock =
 		PTHREAD_MUTEX_INITIALIZER;
 
+static const char *
+lttng_viewer_next_index_return_code_str(enum lttng_viewer_next_index_return_code code)
+{
+	switch (code) {
+	case LTTNG_VIEWER_INDEX_OK:
+		return "INDEX_OK";
+	case LTTNG_VIEWER_INDEX_RETRY:
+		return "INDEX_RETRY";
+	case LTTNG_VIEWER_INDEX_HUP:
+		return "INDEX_HUP";
+	case LTTNG_VIEWER_INDEX_ERR:
+		return "INDEX_ERR";
+	case LTTNG_VIEWER_INDEX_INACTIVE:
+		return "INDEX_INACTIVE";
+	case LTTNG_VIEWER_INDEX_EOF:
+		return "INDEX_EOF";
+	default:
+		abort();
+	}
+}
+
 /*
  * Cleanup the daemon
  */
@@ -156,28 +177,30 @@ static
 int check_new_streams(struct relay_connection *conn)
 {
 	struct relay_session *session;
-	unsigned long current_val;
 	int ret = 0;
 
+	rcu_read_lock();
 	if (!conn->viewer_session) {
 		goto end;
 	}
-	rcu_read_lock();
-	cds_list_for_each_entry_rcu(session,
-			&conn->viewer_session->session_list,
-			viewer_session_node) {
+
+	cds_list_for_each_entry_rcu(
+		session, &conn->viewer_session->session_list, viewer_session_node)
+	{
 		if (!session_get(session)) {
 			continue;
 		}
-		current_val = uatomic_cmpxchg(&session->new_streams, 1, 0);
-		ret = current_val;
+
+		ret = uatomic_read(&session->new_streams);
 		session_put(session);
 		if (ret == 1) {
 			goto end;
 		}
 	}
+
 end:
 	rcu_read_unlock();
+	DBG("Viewer connection has%s new streams: socket_fd = %d", ret == 0 ? " no" : "", conn->sock->fd);
 	return ret;
 }
 
@@ -192,7 +215,6 @@ ssize_t send_viewer_streams(struct lttcomm_sock *sock,
 		uint64_t session_id, unsigned int ignore_sent_flag)
 {
 	ssize_t ret;
-	struct lttng_viewer_stream send_stream;
 	struct lttng_ht_iter iter;
 	struct relay_viewer_stream *vstream;
 
@@ -201,6 +223,7 @@ ssize_t send_viewer_streams(struct lttcomm_sock *sock,
 	cds_lfht_for_each_entry(viewer_streams_ht->ht, &iter.iter, vstream,
 			stream_n.node) {
 		struct ctf_trace *ctf_trace;
+		struct lttng_viewer_stream send_stream = {};
 
 		health_code_update();
 
@@ -268,8 +291,8 @@ end_unlock:
  *
  * Return 0 on success or else a negative value.
  */
-static int make_viewer_streams(struct relay_session *session,
-		struct lttng_trace_chunk *viewer_trace_chunk,
+static int make_viewer_streams(struct relay_session *relay_session,
+		struct relay_viewer_session *viewer_session,
 		enum lttng_viewer_seek seek_t,
 		uint32_t *nb_total,
 		uint32_t *nb_unsent,
@@ -279,18 +302,12 @@ static int make_viewer_streams(struct relay_session *session,
 	int ret;
 	struct lttng_ht_iter iter;
 	struct ctf_trace *ctf_trace;
+	struct relay_stream *relay_stream = NULL;
 
-	assert(session);
-	ASSERT_LOCKED(session->lock);
+	assert(relay_session);
+	ASSERT_LOCKED(relay_session->lock);
 
-	if (!viewer_trace_chunk) {
-		ERR("Internal error: viewer session associated with session \"%s\" has a NULL trace chunk",
-				session->session_name);
-		ret = -1;
-		goto error;
-	}
-
-	if (session->connection_closed) {
+	if (relay_session->connection_closed) {
 		*closed = true;
 	}
 
@@ -299,10 +316,9 @@ static int make_viewer_streams(struct relay_session *session,
 	 * used for a the given session id only.
 	 */
 	rcu_read_lock();
-	cds_lfht_for_each_entry(session->ctf_traces_ht->ht, &iter.iter, ctf_trace,
-			node.node) {
+	cds_lfht_for_each_entry (relay_session->ctf_traces_ht->ht, &iter.iter,
+			ctf_trace, node.node) {
 		bool trace_has_metadata_stream = false;
-		struct relay_stream *stream;
 
 		health_code_update();
 
@@ -314,15 +330,23 @@ static int make_viewer_streams(struct relay_session *session,
 		 * Iterate over all the streams of the trace to see if we have a
 		 * metadata stream.
 		 */
-		cds_list_for_each_entry_rcu(
-				stream, &ctf_trace->stream_list, stream_node)
+		cds_list_for_each_entry_rcu(relay_stream,
+				&ctf_trace->stream_list, stream_node)
 		{
-			if (stream->is_metadata) {
+			bool is_metadata_stream;
+
+			pthread_mutex_lock(&relay_stream->lock);
+			is_metadata_stream = relay_stream->is_metadata;
+			pthread_mutex_unlock(&relay_stream->lock);
+
+			if (is_metadata_stream) {
 				trace_has_metadata_stream = true;
 				break;
 			}
 		}
 
+		relay_stream = NULL;
+
 		/*
 		 * If there is no metadata stream in this trace at the moment
 		 * and we never sent one to the viewer, skip the trace. We
@@ -334,35 +358,102 @@ static int make_viewer_streams(struct relay_session *session,
 			continue;
 		}
 
-		cds_list_for_each_entry_rcu(stream, &ctf_trace->stream_list, stream_node) {
-			struct relay_viewer_stream *vstream;
+		cds_list_for_each_entry_rcu(relay_stream,
+				&ctf_trace->stream_list, stream_node)
+		{
+			struct relay_viewer_stream *viewer_stream;
 
-			if (!stream_get(stream)) {
+			if (!stream_get(relay_stream)) {
 				continue;
 			}
+
+			pthread_mutex_lock(&relay_stream->lock);
 			/*
 			 * stream published is protected by the session lock.
 			 */
-			if (!stream->published) {
+			if (!relay_stream->published) {
 				goto next;
 			}
-			vstream = viewer_stream_get_by_id(stream->stream_handle);
-			if (!vstream) {
+			viewer_stream = viewer_stream_get_by_id(
+					relay_stream->stream_handle);
+			if (!viewer_stream) {
+				struct lttng_trace_chunk *viewer_stream_trace_chunk = NULL;
+
 				/*
 				 * Save that we sent the metadata stream to the
 				 * viewer. So that we know what trace the viewer
 				 * is aware of.
 				 */
-				if (stream->is_metadata) {
-					ctf_trace->metadata_stream_sent_to_viewer =
-							true;
+				if (relay_stream->is_metadata) {
+					ctf_trace->metadata_stream_sent_to_viewer = true;
 				}
-				vstream = viewer_stream_create(stream,
-						viewer_trace_chunk, seek_t);
-				if (!vstream) {
+
+				/*
+				 * If a rotation is ongoing, use a copy of the
+				 * relay stream's chunk to ensure the stream
+				 * files exist.
+				 *
+				 * Otherwise, the viewer session's current trace
+				 * chunk can be used safely.
+				 */
+				if ((relay_stream->ongoing_rotation.is_set ||
+						session_has_ongoing_rotation(relay_session)) &&
+						relay_stream->trace_chunk) {
+					viewer_stream_trace_chunk = lttng_trace_chunk_copy(
+							relay_stream->trace_chunk);
+					if (!viewer_stream_trace_chunk) {
+						ret = -1;
+						ctf_trace_put(ctf_trace);
+						goto error_unlock;
+					}
+				} else {
+					/*
+					 * Transition the viewer session into the newest trace chunk available.
+					 */
+					if (!lttng_trace_chunk_ids_equal(viewer_session->current_trace_chunk,
+							relay_stream->trace_chunk)) {
+
+						ret = viewer_session_set_trace_chunk_copy(
+								viewer_session,
+								relay_stream->trace_chunk);
+						if (ret) {
+							ret = -1;
+							ctf_trace_put(ctf_trace);
+							goto error_unlock;
+						}
+					}
+
+					if (relay_stream->trace_chunk) {
+						/*
+						 * If the corresponding relay
+						 * stream's trace chunk is set,
+						 * the viewer stream will be
+						 * created under it.
+						 *
+						 * Note that a relay stream can
+						 * have a NULL output trace
+						 * chunk (for instance, after a
+						 * clear against a stopped
+						 * session).
+						 */
+						const bool reference_acquired = lttng_trace_chunk_get(
+								viewer_session->current_trace_chunk);
+
+						assert(reference_acquired);
+						viewer_stream_trace_chunk =
+								viewer_session->current_trace_chunk;
+					}
+				}
+
+				viewer_stream = viewer_stream_create(
+						relay_stream,
+						viewer_stream_trace_chunk,
+						seek_t);
+				lttng_trace_chunk_put(viewer_stream_trace_chunk);
+				viewer_stream_trace_chunk = NULL;
+				if (!viewer_stream) {
 					ret = -1;
 					ctf_trace_put(ctf_trace);
-					stream_put(stream);
 					goto error_unlock;
 				}
 
@@ -374,36 +465,40 @@ static int make_viewer_streams(struct relay_session *session,
 				 * Ensure a self-reference is preserved even
 				 * after we have put our local reference.
 				 */
-				if (!viewer_stream_get(vstream)) {
+				if (!viewer_stream_get(viewer_stream)) {
 					ERR("Unable to get self-reference on viewer stream, logic error.");
 					abort();
 				}
 			} else {
-				if (!vstream->sent_flag && nb_unsent) {
+				if (!viewer_stream->sent_flag && nb_unsent) {
 					/* Update number of unsent stream counter. */
 					(*nb_unsent)++;
 				}
 			}
 			/* Update number of total stream counter. */
 			if (nb_total) {
-				if (stream->is_metadata) {
-					if (!stream->closed ||
-							stream->metadata_received > vstream->metadata_sent) {
+				if (relay_stream->is_metadata) {
+					if (!relay_stream->closed ||
+							relay_stream->metadata_received >
+									viewer_stream->metadata_sent) {
 						(*nb_total)++;
 					}
 				} else {
-					if (!stream->closed ||
-						!(((int64_t) (stream->prev_data_seq - stream->last_net_seq_num)) >= 0)) {
-
+					if (!relay_stream->closed ||
+							!(((int64_t)(relay_stream->prev_data_seq -
+									  relay_stream->last_net_seq_num)) >=
+									0)) {
 						(*nb_total)++;
 					}
 				}
 			}
 			/* Put local reference. */
-			viewer_stream_put(vstream);
+			viewer_stream_put(viewer_stream);
 		next:
-			stream_put(stream);
+			pthread_mutex_unlock(&relay_stream->lock);
+			stream_put(relay_stream);
 		}
+		relay_stream = NULL;
 		ctf_trace_put(ctf_trace);
 	}
 
@@ -411,7 +506,12 @@ static int make_viewer_streams(struct relay_session *session,
 
 error_unlock:
 	rcu_read_unlock();
-error:
+
+	if (relay_stream) {
+		pthread_mutex_unlock(&relay_stream->lock);
+		stream_put(relay_stream);
+	}
+
 	return ret;
 }
 
@@ -964,14 +1064,6 @@ int viewer_list_sessions(struct relay_connection *conn)
 			/* Skip closed session */
 			goto next_session;
 		}
-		if (!session->current_trace_chunk) {
-			/*
-			 * Skip un-attachable session. It is either
-			 * being destroyed or has not had a trace
-			 * chunk created against it yet.
-			 */
-			goto next_session;
-		}
 
 		if (count >= buf_count) {
 			struct lttng_viewer_session *newbuf;
@@ -1086,14 +1178,40 @@ int viewer_get_new_streams(struct relay_connection *conn)
 		goto send_reply;
 	}
 
+	/*
+	 * For any new stream, create it with LTTNG_VIEWER_SEEK_BEGINNING since
+	 * that at this point the client is already attached to the session.Aany
+	 * initial stream will have been created with the seek type at attach
+	 * time (for now most readers use the LTTNG_VIEWER_SEEK_LAST on attach).
+	 * Otherwise any event happening in a new stream between the attach and
+	 * a call to viewer_get_new_streams will be "lost" (never received) from
+	 * the viewer's point of view.
+	 */
 	pthread_mutex_lock(&session->lock);
+	/*
+	 * If a session rotation is ongoing, do not attempt to open any
+	 * stream, because the chunk can be in an intermediate state
+	 * due to directory renaming.
+	 */
+	if (session_has_ongoing_rotation(session)) {
+		DBG("Relay session %" PRIu64 " rotation ongoing", session_id);
+		response.status = htobe32(LTTNG_VIEWER_NEW_STREAMS_NO_NEW);
+		goto send_reply_unlock;
+	}
 	ret = make_viewer_streams(session,
-			conn->viewer_session->current_trace_chunk,
-			LTTNG_VIEWER_SEEK_LAST, &nb_total, &nb_unsent,
+			conn->viewer_session,
+			LTTNG_VIEWER_SEEK_BEGINNING, &nb_total, &nb_unsent,
 			&nb_created, &closed);
 	if (ret < 0) {
-		goto error_unlock_session;
+		/*
+		 * This is caused by an internal error; propagate the negative
+		 * 'ret' to close the connection.
+		 */
+		response.status = htobe32(LTTNG_VIEWER_NEW_STREAMS_ERR);
+		goto send_reply_unlock;
 	}
+
+	uatomic_set(&session->new_streams, 0);
 	send_streams = 1;
 	response.status = htobe32(LTTNG_VIEWER_NEW_STREAMS_OK);
 
@@ -1147,10 +1265,6 @@ end_put_session:
 	}
 error:
 	return ret;
-error_unlock_session:
-	pthread_mutex_unlock(&session->lock);
-	session_put(session);
-	return ret;
 }
 
 /*
@@ -1200,15 +1314,6 @@ int viewer_attach_session(struct relay_connection *conn)
 	DBG("Attach session ID %" PRIu64 " received", session_id);
 
 	pthread_mutex_lock(&session->lock);
-	if (!session->current_trace_chunk) {
-		/*
-		 * Session is either being destroyed or it never had a trace
-		 * chunk created against it.
-		 */
-		DBG("Session requested by live client has no current trace chunk, returning unknown session");
-		response.status = htobe32(LTTNG_VIEWER_ATTACH_UNK);
-		goto send_reply;
-	}
 	if (session->live_timer == 0) {
 		DBG("Not live session");
 		response.status = htobe32(LTTNG_VIEWER_ATTACH_NOT_LIVE);
@@ -1236,8 +1341,19 @@ int viewer_attach_session(struct relay_connection *conn)
 		goto send_reply;
 	}
 
+	/*
+	 * If a session rotation is ongoing, do not attempt to open any
+	 * stream, because the chunk can be in an intermediate state
+	 * due to directory renaming.
+	 */
+	if (session_has_ongoing_rotation(session)) {
+		DBG("Relay session %" PRIu64 " rotation ongoing", session_id);
+		send_streams = 0;
+		goto send_reply;
+	}
+
 	ret = make_viewer_streams(session,
-			conn->viewer_session->current_trace_chunk, seek_type,
+			conn->viewer_session, seek_type,
 			&nb_streams, NULL, NULL, &closed);
 	if (ret < 0) {
 		goto end_put_session;
@@ -1317,10 +1433,12 @@ static int try_open_index(struct relay_viewer_stream *vstream,
 	/*
 	 * First time, we open the index file and at least one index is ready.
 	 */
-	if (rstream->index_received_seqcount == 0) {
+	if (rstream->index_received_seqcount == 0 ||
+			!vstream->stream_file.trace_chunk) {
 		ret = -ENOENT;
 		goto end;
 	}
+
 	chunk_status = lttng_index_file_create_from_trace_chunk_read_only(
 			vstream->stream_file.trace_chunk, rstream->path_name,
 			rstream->channel_name, rstream->tracefile_size,
@@ -1474,6 +1592,24 @@ index_ready:
 	return 1;
 }
 
+static
+void viewer_stream_rotate_to_trace_chunk(struct relay_viewer_stream *vstream,
+		 struct lttng_trace_chunk *new_trace_chunk)
+{
+	lttng_trace_chunk_put(vstream->stream_file.trace_chunk);
+
+	if (new_trace_chunk) {
+		const bool acquired_reference = lttng_trace_chunk_get(
+				new_trace_chunk);
+
+		assert(acquired_reference);
+	}
+
+	vstream->stream_file.trace_chunk = new_trace_chunk;
+	viewer_stream_sync_tracefile_array_tail(vstream);
+	viewer_stream_close_files(vstream);
+}
+
 /*
  * Send the next index for a stream.
  *
@@ -1490,6 +1626,7 @@ int viewer_get_next_index(struct relay_connection *conn)
 	struct relay_stream *rstream = NULL;
 	struct ctf_trace *ctf_trace = NULL;
 	struct relay_viewer_stream *metadata_viewer_stream = NULL;
+	bool attached_sessions_have_new_streams = false;
 
 	assert(conn);
 
@@ -1520,6 +1657,13 @@ int viewer_get_next_index(struct relay_connection *conn)
 	metadata_viewer_stream =
 			ctf_trace_get_viewer_metadata_stream(ctf_trace);
 
+	/*
+	 * Hold the session lock to protect against concurrent changes
+	 * to the chunk files (e.g. rename done by clear), which are
+	 * protected by the session ongoing rotation state. Those are
+	 * synchronized with the session lock.
+	 */
+	pthread_mutex_lock(&rstream->trace->session->lock);
 	pthread_mutex_lock(&rstream->lock);
 
 	/*
@@ -1530,70 +1674,67 @@ int viewer_get_next_index(struct relay_connection *conn)
 		goto send_reply;
 	}
 
+	ret = check_new_streams(conn);
+	if (ret < 0) {
+		viewer_index.status = LTTNG_VIEWER_INDEX_ERR;
+		ERR("Error checking for new streams in the attached sessions, returning status=%s",
+		    lttng_viewer_next_index_return_code_str(
+			    (enum lttng_viewer_next_index_return_code) viewer_index.status));
+		goto send_reply;
+	} else if (ret == 1) {
+		attached_sessions_have_new_streams = true;
+	}
+
 	if (rstream->ongoing_rotation.is_set) {
 		/* Rotation is ongoing, try again later. */
 		viewer_index.status = htobe32(LTTNG_VIEWER_INDEX_RETRY);
 		goto send_reply;
 	}
 
-	if (rstream->trace->session->ongoing_rotation) {
+	if (session_has_ongoing_rotation(rstream->trace->session)) {
 		/* Rotation is ongoing, try again later. */
 		viewer_index.status = htobe32(LTTNG_VIEWER_INDEX_RETRY);
 		goto send_reply;
 	}
 
-	if (rstream->trace_chunk) {
-		uint64_t rchunk_id, vchunk_id;
+	/*
+	 * Transition the viewer session into the newest trace chunk available.
+	 */
+	if (!lttng_trace_chunk_ids_equal(
+			conn->viewer_session->current_trace_chunk,
+			rstream->trace_chunk)) {
+		DBG("Relay stream and viewer chunk ids differ");
 
-		/*
-		 * If the relay stream is not yet closed, ensure the viewer
-		 * chunk matches the relay chunk after clear.
-		 */
-		if (lttng_trace_chunk_get_id(rstream->trace_chunk,
-				&rchunk_id) != LTTNG_TRACE_CHUNK_STATUS_OK) {
+		ret = viewer_session_set_trace_chunk_copy(
+				conn->viewer_session,
+				rstream->trace_chunk);
+		if (ret) {
 			viewer_index.status = htobe32(LTTNG_VIEWER_INDEX_ERR);
 			goto send_reply;
 		}
-		if (lttng_trace_chunk_get_id(
-				conn->viewer_session->current_trace_chunk,
-				&vchunk_id) != LTTNG_TRACE_CHUNK_STATUS_OK) {
-			viewer_index.status = htobe32(LTTNG_VIEWER_INDEX_ERR);
-			goto send_reply;
-		}
-
-		if (rchunk_id != vchunk_id) {
-			DBG("Relay and viewer chunk ids differ: "
-				"rchunk_id %" PRIu64 " vchunk_id %" PRIu64,
-				rchunk_id, vchunk_id);
-
-			lttng_trace_chunk_put(
-				conn->viewer_session->current_trace_chunk);
-			conn->viewer_session->current_trace_chunk = NULL;
-			ret = viewer_session_set_trace_chunk_copy(
-					conn->viewer_session,
-					rstream->trace_chunk);
-			if (ret) {
-				viewer_index.status =
-					htobe32(LTTNG_VIEWER_INDEX_ERR);
-				goto send_reply;
-			}
-		}
 	}
-	if (conn->viewer_session->current_trace_chunk !=
-			vstream->stream_file.trace_chunk) {
-		bool acquired_reference;
 
-		DBG("Viewer session and viewer stream chunk differ: "
+	/*
+	 * Transition the viewer stream into the latest trace chunk available.
+	 *
+	 * Note that the stream must _not_ rotate in one precise condition:
+	 * the relay stream has rotated to a NULL trace chunk and the viewer
+	 * stream is consuming the trace chunk that was active just before
+	 * that rotation to NULL.
+	 *
+	 * This allows clients to consume all the packets of a trace chunk
+	 * after a session's destruction.
+	 */
+	if (!lttng_trace_chunk_ids_equal(conn->viewer_session->current_trace_chunk, vstream->stream_file.trace_chunk) &&
+			!(rstream->completed_rotation_count == vstream->last_seen_rotation_count + 1 && !rstream->trace_chunk)) {
+		DBG("Viewer session and viewer stream chunk IDs differ: "
 				"vsession chunk %p vstream chunk %p",
 				conn->viewer_session->current_trace_chunk,
 				vstream->stream_file.trace_chunk);
-		lttng_trace_chunk_put(vstream->stream_file.trace_chunk);
-		acquired_reference = lttng_trace_chunk_get(conn->viewer_session->current_trace_chunk);
-		assert(acquired_reference);
-		vstream->stream_file.trace_chunk =
-			conn->viewer_session->current_trace_chunk;
-		viewer_stream_sync_tracefile_array_tail(vstream);
-		viewer_stream_close_files(vstream);
+		viewer_stream_rotate_to_trace_chunk(vstream,
+				conn->viewer_session->current_trace_chunk);
+		vstream->last_seen_rotation_count =
+				rstream->completed_rotation_count;
 	}
 
 	ret = check_index_status(vstream, rstream, ctf_trace, &viewer_index);
@@ -1606,6 +1747,7 @@ int viewer_get_next_index(struct relay_connection *conn)
 		 */
 		goto send_reply;
 	}
+
 	/* At this point, ret is 0 thus we will be able to read the index. */
 	assert(!ret);
 
@@ -1665,14 +1807,6 @@ int viewer_get_next_index(struct relay_connection *conn)
 		vstream->stream_file.handle = fs_handle;
 	}
 
-	ret = check_new_streams(conn);
-	if (ret < 0) {
-		viewer_index.status = htobe32(LTTNG_VIEWER_INDEX_ERR);
-		goto send_reply;
-	} else if (ret == 1) {
-		viewer_index.flags |= LTTNG_VIEWER_FLAG_NEW_STREAM;
-	}
-
 	ret = lttng_index_file_read(vstream->index_file, &packet_index);
 	if (ret) {
 		ERR("Relay error reading index file");
@@ -1700,6 +1834,7 @@ int viewer_get_next_index(struct relay_connection *conn)
 send_reply:
 	if (rstream) {
 		pthread_mutex_unlock(&rstream->lock);
+		pthread_mutex_unlock(&rstream->trace->session->lock);
 	}
 
 	if (metadata_viewer_stream) {
@@ -1716,6 +1851,10 @@ send_reply:
 		pthread_mutex_unlock(&metadata_viewer_stream->stream->lock);
 	}
 
+	if (attached_sessions_have_new_streams) {
+		viewer_index.flags |= LTTNG_VIEWER_FLAG_NEW_STREAM;
+	}
+
 	viewer_index.flags = htobe32(viewer_index.flags);
 	health_code_update();
 
@@ -1741,6 +1880,7 @@ end:
 
 error_put:
 	pthread_mutex_unlock(&rstream->lock);
+	pthread_mutex_unlock(&rstream->trace->session->lock);
 	if (metadata_viewer_stream) {
 		viewer_stream_put(metadata_viewer_stream);
 	}
@@ -1876,6 +2016,7 @@ int viewer_get_metadata(struct relay_connection *conn)
 	struct lttng_viewer_get_metadata request;
 	struct lttng_viewer_metadata_packet reply;
 	struct relay_viewer_stream *vstream = NULL;
+	bool dispose_of_stream = false;
 
 	assert(conn);
 
@@ -1906,6 +2047,9 @@ int viewer_get_metadata(struct relay_connection *conn)
 		reply.status = htobe32(LTTNG_VIEWER_METADATA_ERR);
 		goto send_reply;
 	}
+
+	pthread_mutex_lock(&vstream->stream->trace->session->lock);
+	pthread_mutex_lock(&vstream->stream->trace->lock);
 	pthread_mutex_lock(&vstream->stream->lock);
 	if (!vstream->stream->is_metadata) {
 		ERR("Invalid metadata stream");
@@ -1914,11 +2058,7 @@ int viewer_get_metadata(struct relay_connection *conn)
 
 	if (vstream->metadata_sent >= vstream->stream->metadata_received) {
 		/*
-		 * The live viewers expect to receive a NO_NEW_METADATA
-		 * status before a stream disappears, otherwise they abort the
-		 * entire live connection when receiving an error status.
-		 *
-		 * Clear feature resets the metadata_sent to 0 until the
+		 * Clear feature resets the metadata_received to 0 until the
 		 * same metadata is received again.
 		 */
 		reply.status = htobe32(LTTNG_VIEWER_NO_NEW_METADATA);
@@ -1926,20 +2066,68 @@ int viewer_get_metadata(struct relay_connection *conn)
 		 * The live viewer considers a closed 0 byte metadata stream as
 		 * an error.
 		 */
-		if (vstream->metadata_sent > 0) {
-			vstream->stream->no_new_metadata_notified = true;
-			if (vstream->stream->closed) {
-				/* Release ownership for the viewer metadata stream. */
-				viewer_stream_put(vstream);
-			}
-		}
+		dispose_of_stream = vstream->metadata_sent > 0 && vstream->stream->closed;
 		goto send_reply;
 	}
 
+	if (vstream->stream->trace_chunk &&
+			!lttng_trace_chunk_ids_equal(
+				conn->viewer_session->current_trace_chunk,
+				vstream->stream->trace_chunk)) {
+		/* A rotation has occurred on the relay stream. */
+		DBG("Metadata relay stream and viewer chunk ids differ");
+
+		ret = viewer_session_set_trace_chunk_copy(
+				conn->viewer_session,
+				vstream->stream->trace_chunk);
+		if (ret) {
+			reply.status = htobe32(LTTNG_VIEWER_METADATA_ERR);
+			goto send_reply;
+		}
+	}
+
+	if (conn->viewer_session->current_trace_chunk &&
+			!lttng_trace_chunk_ids_equal(conn->viewer_session->current_trace_chunk,
+					vstream->stream_file.trace_chunk)) {
+		bool acquired_reference;
+
+		DBG("Viewer session and viewer stream chunk differ: "
+				"vsession chunk %p vstream chunk %p",
+				conn->viewer_session->current_trace_chunk,
+				vstream->stream_file.trace_chunk);
+		lttng_trace_chunk_put(vstream->stream_file.trace_chunk);
+		acquired_reference = lttng_trace_chunk_get(conn->viewer_session->current_trace_chunk);
+		assert(acquired_reference);
+		vstream->stream_file.trace_chunk =
+			conn->viewer_session->current_trace_chunk;
+		viewer_stream_close_files(vstream);
+	}
+
 	len = vstream->stream->metadata_received - vstream->metadata_sent;
 
-	/* first time, we open the metadata file */
-	if (!vstream->stream_file.handle) {
+	if (!vstream->stream_file.trace_chunk) {
+		if (vstream->stream->trace->session->connection_closed) {
+			/*
+			 * If the connection is closed, there is no way for the metadata stream
+			 * to ever transition back to an active chunk. As such, signal to the viewer
+			 * that there is no new metadata available.
+			 *
+			 * The stream can be disposed-of. On the next execution of this command,
+			 * the relay daemon will reply with an error status since the stream can't
+			 * be found.
+			 */
+			dispose_of_stream = true;
+		}
+
+		reply.status = htobe32(LTTNG_VIEWER_NO_NEW_METADATA);
+		len = 0;
+		goto send_reply;
+	} else if (vstream->stream_file.trace_chunk &&
+			!vstream->stream_file.handle && len > 0) {
+		/*
+		 * Either this is the first time the metadata file is read, or a
+		 * rotation of the corresponding relay stream has occurred.
+		 */
 		struct fs_handle *fs_handle;
 		char file_path[LTTNG_PATH_MAX];
 		enum lttng_trace_chunk_status status;
@@ -1974,6 +2162,33 @@ int viewer_get_metadata(struct relay_connection *conn)
 			goto error;
 		}
 		vstream->stream_file.handle = fs_handle;
+
+		if (vstream->metadata_sent != 0) {
+			/*
+			 * The client does not expect to receive any metadata
+			 * it has received and metadata files in successive
+			 * chunks must be a strict superset of one another.
+			 *
+			 * Skip the first `metadata_sent` bytes to ensure
+			 * they are not sent a second time to the client.
+			 *
+			 * Baring a block layer error or an internal error,
+			 * this seek should not fail as
+			 * `vstream->stream->metadata_received` is reset when
+			 * a relay stream is rotated. If this is reached, it is
+			 * safe to assume that
+			 * `metadata_received` > `metadata_sent`.
+			 */
+			const off_t seek_ret = fs_handle_seek(fs_handle,
+					vstream->metadata_sent, SEEK_SET);
+
+			if (seek_ret < 0) {
+				PERROR("Failed to seek metadata viewer stream file to `sent` position: pos = %" PRId64,
+						vstream->metadata_sent);
+				reply.status = htobe32(LTTNG_VIEWER_METADATA_ERR);
+				goto send_reply;
+			}
+		}
 	}
 
 	reply.len = htobe64(len);
@@ -1992,8 +2207,34 @@ int viewer_get_metadata(struct relay_connection *conn)
 	fs_handle_put_fd(vstream->stream_file.handle);
 	fd = -1;
 	if (read_len < len) {
-		PERROR("Relay reading metadata file");
-		goto error;
+		if (read_len < 0) {
+			PERROR("Failed to read metadata file");
+			goto error;
+		} else {
+			/*
+			 * A clear has been performed which prevents the relay
+			 * from sending `len` bytes of metadata.
+			 *
+			 * It is important not to send any metadata if we
+			 * couldn't read all the available metadata in one shot:
+			 * sending partial metadata can cause the client to
+			 * attempt to parse an incomplete (incoherent) metadata
+			 * stream, which would result in an error.
+			 */
+			const off_t seek_ret = fs_handle_seek(
+					vstream->stream_file.handle, -read_len,
+					SEEK_CUR);
+
+			DBG("Failed to read metadata: requested = %" PRIu64 ", got = %zd",
+					len, read_len);
+			read_len = 0;
+			len = 0;
+			if (seek_ret < 0) {
+				PERROR("Failed to restore metadata file position after partial read");
+				ret = -1;
+				goto error;
+			}
+		}
 	}
 	vstream->metadata_sent += read_len;
 	reply.status = htobe32(LTTNG_VIEWER_METADATA_OK);
@@ -2007,6 +2248,8 @@ send_reply:
 	health_code_update();
 	if (vstream) {
 		pthread_mutex_unlock(&vstream->stream->lock);
+		pthread_mutex_unlock(&vstream->stream->trace->lock);
+		pthread_mutex_unlock(&vstream->stream->trace->session->lock);
 	}
 	ret = send_response(conn->sock, &reply, sizeof(reply));
 	if (ret < 0) {
@@ -2031,7 +2274,22 @@ end_free:
 end:
 	if (vstream) {
 		viewer_stream_put(vstream);
+		if (dispose_of_stream) {
+			/*
+			 * Trigger the destruction of the viewer stream
+			 * by releasing its global reference.
+			 *
+			 * The live viewers expect to receive a NO_NEW_METADATA
+			 * status before a stream disappears, otherwise they abort the
+			 * entire live connection when receiving an error status.
+			 *
+			 * On the next query for this stream, an error will be reported to the
+			 * client.
+			 */
+			viewer_stream_put(vstream);
+		}
 	}
+
 	return ret;
 }
 
