@@ -1,19 +1,9 @@
 /*
- * Copyright (C) 2011 - David Goulet <david.goulet@polymtl.ca>
- * Copyright (C) 2016 - Jérémie Galarneau <jeremie.galarneau@efficios.com>
+ * Copyright (C) 2011 EfficiOS Inc.
+ * Copyright (C) 2016 Jérémie Galarneau <jeremie.galarneau@efficios.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #define _LGPL_SOURCE
@@ -28,7 +18,8 @@
 #include "channel.h"
 #include "lttng-sessiond.h"
 #include "kernel.h"
-#include "ust-ctl.h"
+#include "lttng-ust-ctl.h"
+#include "lttng-ust-error.h"
 #include "utils.h"
 #include "ust-app.h"
 #include "agent.h"
@@ -41,6 +32,7 @@ struct lttng_channel *channel_new_default_attr(int dom,
 {
 	struct lttng_channel *chan;
 	const char *channel_name = DEFAULT_CHANNEL_NAME;
+	struct lttng_channel_extended *extended_attr = NULL;
 
 	chan = zmalloc(sizeof(struct lttng_channel));
 	if (chan == NULL) {
@@ -48,6 +40,14 @@ struct lttng_channel *channel_new_default_attr(int dom,
 		goto error_alloc;
 	}
 
+	extended_attr = zmalloc(sizeof(struct lttng_channel_extended));
+	if (!extended_attr) {
+		PERROR("zmalloc channel extended init");
+		goto error;
+	}
+
+	chan->attr.extended.ptr = extended_attr;
+
 	/* Same for all domains. */
 	chan->attr.overwrite = DEFAULT_CHANNEL_OVERWRITE;
 	chan->attr.tracefile_size = DEFAULT_CHANNEL_TRACEFILE_SIZE;
@@ -63,6 +63,9 @@ struct lttng_channel *channel_new_default_attr(int dom,
 		chan->attr.switch_timer_interval = DEFAULT_KERNEL_CHANNEL_SWITCH_TIMER;
 		chan->attr.read_timer_interval = DEFAULT_KERNEL_CHANNEL_READ_TIMER;
 		chan->attr.live_timer_interval = DEFAULT_KERNEL_CHANNEL_LIVE_TIMER;
+		extended_attr->blocking_timeout = DEFAULT_KERNEL_CHANNEL_BLOCKING_TIMEOUT;
+		extended_attr->monitor_timer_interval =
+			DEFAULT_KERNEL_CHANNEL_MONITOR_TIMER;
 		break;
 	case LTTNG_DOMAIN_JUL:
 		channel_name = DEFAULT_JUL_CHANNEL_NAME;
@@ -86,6 +89,9 @@ common_ust:
 				DEFAULT_UST_UID_CHANNEL_READ_TIMER;
 			chan->attr.live_timer_interval =
 				DEFAULT_UST_UID_CHANNEL_LIVE_TIMER;
+			extended_attr->blocking_timeout = DEFAULT_UST_UID_CHANNEL_BLOCKING_TIMEOUT;
+			extended_attr->monitor_timer_interval =
+				DEFAULT_UST_UID_CHANNEL_MONITOR_TIMER;
 			break;
 		case LTTNG_BUFFER_PER_PID:
 		default:
@@ -97,7 +103,10 @@ common_ust:
 			chan->attr.read_timer_interval =
 				DEFAULT_UST_PID_CHANNEL_READ_TIMER;
 			chan->attr.live_timer_interval =
-				DEFAULT_UST_UID_CHANNEL_LIVE_TIMER;
+				DEFAULT_UST_PID_CHANNEL_LIVE_TIMER;
+			extended_attr->blocking_timeout = DEFAULT_UST_PID_CHANNEL_BLOCKING_TIMEOUT;
+			extended_attr->monitor_timer_interval =
+				DEFAULT_UST_PID_CHANNEL_MONITOR_TIMER;
 			break;
 		}
 		break;
@@ -113,11 +122,21 @@ common_ust:
 	return chan;
 
 error:
+	free(extended_attr);
 	free(chan);
 error_alloc:
 	return NULL;
 }
 
+void channel_attr_destroy(struct lttng_channel *channel)
+{
+	if (!channel) {
+		return;
+	}
+	free(channel->attr.extended.ptr);
+	free(channel);
+}
+
 /*
  * Disable kernel channel of the kernel session.
  */
@@ -192,6 +211,15 @@ static int channel_validate(struct lttng_channel *attr)
 	return 0;
 }
 
+static int channel_validate_kernel(struct lttng_channel *attr)
+{
+	/* Kernel channels do not support blocking timeout. */
+	if (((struct lttng_channel_extended *)attr->attr.extended.ptr)->blocking_timeout) {
+		return -1;
+	}
+	return 0;
+}
+
 /*
  * Create kernel channel of the kernel session and notify kernel thread.
  */
@@ -222,17 +250,17 @@ int channel_kernel_create(struct ltt_kernel_session *ksession,
 		attr->attr.overwrite = !!ksession->snapshot_mode;
 	}
 
-	/* Enforce mmap output for snapshot sessions. */
-	if (ksession->snapshot_mode) {
-		attr->attr.output = LTTNG_EVENT_MMAP;
-	}
-
 	/* Validate common channel properties. */
 	if (channel_validate(attr) < 0) {
 		ret = LTTNG_ERR_INVALID;
 		goto error;
 	}
 
+	if (channel_validate_kernel(attr) < 0) {
+		ret = LTTNG_ERR_INVALID;
+		goto error;
+	}
+
 	/* Channel not found, creating it */
 	ret = kernel_create_channel(ksession, attr);
 	if (ret < 0) {
@@ -249,7 +277,7 @@ int channel_kernel_create(struct ltt_kernel_session *ksession,
 
 	ret = LTTNG_OK;
 error:
-	free(defattr);
+	channel_attr_destroy(defattr);
 	return ret;
 }
 
@@ -269,6 +297,18 @@ int channel_ust_enable(struct ltt_ust_session *usess,
 		DBG3("Channel %s already enabled. Skipping", uchan->name);
 		ret = LTTNG_ERR_UST_CHAN_EXIST;
 		goto end;
+	} else {
+		uchan->enabled = 1;
+		DBG2("Channel %s enabled successfully", uchan->name);
+	}
+
+	if (!usess->active) {
+		/*
+		 * The channel will be activated against the apps
+		 * when the session is started as part of the
+		 * application channel "synchronize" operation.
+		 */
+		goto end;
 	}
 
 	DBG2("Channel %s being enabled in UST domain", uchan->name);
@@ -283,8 +323,6 @@ int channel_ust_enable(struct ltt_ust_session *usess,
 	 */
 	(void) ust_app_enable_channel_glb(usess, uchan);
 
-	uchan->enabled = 1;
-	DBG2("Channel %s enabled successfully", uchan->name);
 
 end:
 	return ret;
@@ -423,13 +461,6 @@ int channel_ust_create(struct ltt_ust_session *usess,
 		goto error_free_chan;
 	}
 
-	/* Enable channel for global domain */
-	ret = ust_app_create_channel_glb(usess, uchan);
-	if (ret < 0 && ret != -LTTNG_UST_ERR_EXIST) {
-		ret = LTTNG_ERR_UST_CHAN_FAIL;
-		goto error_free_chan;
-	}
-
 	/* Adding the channel to the channel hash table. */
 	rcu_read_lock();
 	if (strncmp(uchan->name, DEFAULT_METADATA_NAME,
@@ -461,7 +492,7 @@ int channel_ust_create(struct ltt_ust_session *usess,
 		}
 	}
 
-	free(defattr);
+	channel_attr_destroy(defattr);
 	return LTTNG_OK;
 
 error_remove_chan:
@@ -471,7 +502,7 @@ error_remove_chan:
 error_free_chan:
 	trace_ust_destroy_channel(uchan);
 error:
-	free(defattr);
+	channel_attr_destroy(defattr);
 	return ret;
 }
 
@@ -491,6 +522,9 @@ int channel_ust_disable(struct ltt_ust_session *usess,
 		DBG2("Channel UST %s already disabled", uchan->name);
 		goto end;
 	}
+	if (!usess->active) {
+		goto end;
+	}
 
 	DBG2("Channel %s being disabled in UST global domain", uchan->name);
 	/* Disable channel for global domain */
@@ -510,3 +544,56 @@ end:
 error:
 	return ret;
 }
+
+struct lttng_channel *trace_ust_channel_to_lttng_channel(
+		const struct ltt_ust_channel *uchan)
+{
+	struct lttng_channel *channel = NULL, *ret = NULL;
+
+	channel = lttng_channel_create_internal();
+	if (!channel) {
+		goto end;
+	}
+
+	if (lttng_strncpy(channel->name, uchan->name, LTTNG_SYMBOL_NAME_LEN)) {
+		goto end;
+	}
+
+	channel->attr.overwrite = uchan->attr.overwrite;
+	channel->attr.subbuf_size = uchan->attr.subbuf_size;
+	channel->attr.num_subbuf = uchan->attr.num_subbuf;
+	channel->attr.switch_timer_interval = uchan->attr.switch_timer_interval;
+	channel->attr.read_timer_interval = uchan->attr.read_timer_interval;
+	channel->enabled = uchan->enabled;
+	channel->attr.tracefile_size = uchan->tracefile_size;
+	channel->attr.tracefile_count = uchan->tracefile_count;
+
+	/*
+	 * Map enum lttng_ust_output to enum lttng_event_output.
+	 */
+	switch (uchan->attr.output) {
+	case LTTNG_UST_MMAP:
+		channel->attr.output = LTTNG_EVENT_MMAP;
+		break;
+	default:
+		/*
+		 * LTTNG_UST_MMAP is the only supported UST
+		 * output mode.
+		 */
+		abort();
+		break;
+	}
+
+	lttng_channel_set_blocking_timeout(
+			channel, uchan->attr.u.s.blocking_timeout);
+	lttng_channel_set_monitor_timer_interval(
+			channel, uchan->monitor_timer_interval);
+
+	ret = channel;
+	channel = NULL;
+
+end:
+	lttng_channel_destroy(channel);
+
+	return ret;
+}
