@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 David Goulet <david.goulet@polymtl.ca>
+ * Copyright (C) 2011 EfficiOS Inc.
  * Copyright (C) 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  * Copyright (C) 2013 Jérémie Galarneau <jeremie.galarneau@efficios.com>
  *
@@ -23,6 +23,7 @@
 #include <pthread.h>
 #include <signal.h>
 #include <stddef.h>
+#include <stdint.h>
 #include <sys/stat.h>
 
 #include "client.h"
@@ -533,112 +534,6 @@ static unsigned int lttng_sessions_count(uid_t uid, gid_t gid)
 	return i;
 }
 
-static int receive_userspace_probe(struct command_ctx *cmd_ctx, int sock,
-		int *sock_error, struct lttng_event *event)
-{
-	int fd, ret;
-	struct lttng_userspace_probe_location *probe_location;
-	const struct lttng_userspace_probe_location_lookup_method *lookup = NULL;
-	struct lttng_dynamic_buffer probe_location_buffer;
-	struct lttng_buffer_view buffer_view;
-
-	/*
-	 * Create a buffer to store the serialized version of the probe
-	 * location.
-	 */
-	lttng_dynamic_buffer_init(&probe_location_buffer);
-	ret = lttng_dynamic_buffer_set_size(&probe_location_buffer,
-			cmd_ctx->lsm->u.enable.userspace_probe_location_len);
-	if (ret) {
-		ret = LTTNG_ERR_NOMEM;
-		goto error;
-	}
-
-	/*
-	 * Receive the probe location.
-	 */
-	ret = lttcomm_recv_unix_sock(sock, probe_location_buffer.data,
-			probe_location_buffer.size);
-	if (ret <= 0) {
-		DBG("Nothing recv() from client var len data... continuing");
-		*sock_error = 1;
-		lttng_dynamic_buffer_reset(&probe_location_buffer);
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	buffer_view = lttng_buffer_view_from_dynamic_buffer(
-			&probe_location_buffer, 0, probe_location_buffer.size);
-
-	/*
-	 * Extract the probe location from the serialized version.
-	 */
-	ret = lttng_userspace_probe_location_create_from_buffer(
-				&buffer_view, &probe_location);
-	if (ret < 0) {
-		WARN("Failed to create a userspace probe location from the received buffer");
-		lttng_dynamic_buffer_reset( &probe_location_buffer);
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	/*
-	 * Receive the file descriptor to the target binary from the client.
-	 */
-	DBG("Receiving userspace probe target FD from client ...");
-	ret = lttcomm_recv_fds_unix_sock(sock, &fd, 1);
-	if (ret <= 0) {
-		DBG("Nothing recv() from client userspace probe fd... continuing");
-		*sock_error = 1;
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	/*
-	 * Set the file descriptor received from the client through the unix
-	 * socket in the probe location.
-	 */
-	lookup = lttng_userspace_probe_location_get_lookup_method(probe_location);
-	if (!lookup) {
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	/*
-	 * From the kernel tracer's perspective, all userspace probe event types
-	 * are all the same: a file and an offset.
-	 */
-	switch (lttng_userspace_probe_location_lookup_method_get_type(lookup)) {
-	case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_FUNCTION_ELF:
-		ret = lttng_userspace_probe_location_function_set_binary_fd(
-				probe_location, fd);
-		break;
-	case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_TRACEPOINT_SDT:
-		ret = lttng_userspace_probe_location_tracepoint_set_binary_fd(
-				probe_location, fd);
-		break;
-	default:
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	if (ret) {
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	/* Attach the probe location to the event. */
-	ret = lttng_event_set_userspace_probe_location(event, probe_location);
-	if (ret) {
-		ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
-		goto error;
-	}
-
-	lttng_dynamic_buffer_reset(&probe_location_buffer);
-error:
-	return ret;
-}
-
 /*
  * Version of setup_lttng_msg() without command header.
  */
@@ -716,6 +611,9 @@ static int process_client_msg(struct command_ctx *cmd_ctx, int *sock,
 	int ret = LTTNG_OK;
 	int need_tracing_session = 1;
 	int need_domain;
+	struct lttng_dynamic_buffer payload;
+
+	lttng_dynamic_buffer_init(&payload);
 
 	DBG("Processing client command %d", cmd_ctx->lsm->cmd_type);
 
@@ -1083,74 +981,7 @@ skip_domain:
 	switch (cmd_ctx->lsm->cmd_type) {
 	case LTTNG_ADD_CONTEXT:
 	{
-		/*
-		 * An LTTNG_ADD_CONTEXT command might have a supplementary
-		 * payload if the context being added is an application context.
-		 */
-		if (cmd_ctx->lsm->u.context.ctx.ctx ==
-				LTTNG_EVENT_CONTEXT_APP_CONTEXT) {
-			char *provider_name = NULL, *context_name = NULL;
-			size_t provider_name_len =
-					cmd_ctx->lsm->u.context.provider_name_len;
-			size_t context_name_len =
-					cmd_ctx->lsm->u.context.context_name_len;
-
-			if (provider_name_len == 0 || context_name_len == 0) {
-				/*
-				 * Application provider and context names MUST
-				 * be provided.
-				 */
-				ret = -LTTNG_ERR_INVALID;
-				goto error;
-			}
-
-			provider_name = zmalloc(provider_name_len + 1);
-			if (!provider_name) {
-				ret = -LTTNG_ERR_NOMEM;
-				goto error;
-			}
-			cmd_ctx->lsm->u.context.ctx.u.app_ctx.provider_name =
-					provider_name;
-
-			context_name = zmalloc(context_name_len + 1);
-			if (!context_name) {
-				ret = -LTTNG_ERR_NOMEM;
-				goto error_add_context;
-			}
-			cmd_ctx->lsm->u.context.ctx.u.app_ctx.ctx_name =
-					context_name;
-
-			ret = lttcomm_recv_unix_sock(*sock, provider_name,
-					provider_name_len);
-			if (ret < 0) {
-				goto error_add_context;
-			}
-
-			ret = lttcomm_recv_unix_sock(*sock, context_name,
-					context_name_len);
-			if (ret < 0) {
-				goto error_add_context;
-			}
-		}
-
-		/*
-		 * cmd_add_context assumes ownership of the provider and context
-		 * names.
-		 */
-		ret = cmd_add_context(cmd_ctx->session,
-				cmd_ctx->lsm->domain.type,
-				cmd_ctx->lsm->u.context.channel_name,
-				ALIGNED_CONST_PTR(cmd_ctx->lsm->u.context.ctx),
-				kernel_poll_pipe[1]);
-
-		cmd_ctx->lsm->u.context.ctx.u.app_ctx.provider_name = NULL;
-		cmd_ctx->lsm->u.context.ctx.u.app_ctx.ctx_name = NULL;
-error_add_context:
-		free(cmd_ctx->lsm->u.context.ctx.u.app_ctx.provider_name);
-		free(cmd_ctx->lsm->u.context.ctx.u.app_ctx.ctx_name);
-		if (ret < 0) {
-			goto error;
-		}
+		ret = cmd_add_context(cmd_ctx, *sock, kernel_poll_pipe[1]);
 		break;
 	}
 	case LTTNG_DISABLE_CHANNEL:
@@ -1161,46 +992,12 @@ error_add_context:
 	}
 	case LTTNG_DISABLE_EVENT:
 	{
-
-		/*
-		 * FIXME: handle filter; for now we just receive the filter's
-		 * bytecode along with the filter expression which are sent by
-		 * liblttng-ctl and discard them.
-		 *
-		 * This fixes an issue where the client may block while sending
-		 * the filter payload and encounter an error because the session
-		 * daemon closes the socket without ever handling this data.
-		 */
-		size_t count = cmd_ctx->lsm->u.disable.expression_len +
-			cmd_ctx->lsm->u.disable.bytecode_len;
-
-		if (count) {
-			char data[LTTNG_FILTER_MAX_LEN];
-
-			DBG("Discarding disable event command payload of size %zu", count);
-			while (count) {
-				ret = lttcomm_recv_unix_sock(*sock, data,
-				        count > sizeof(data) ? sizeof(data) : count);
-				if (ret < 0) {
-					goto error;
-				}
-
-				count -= (size_t) ret;
-			}
-		}
-		ret = cmd_disable_event(cmd_ctx->session, cmd_ctx->lsm->domain.type,
-				cmd_ctx->lsm->u.disable.channel_name,
-				ALIGNED_CONST_PTR(cmd_ctx->lsm->u.disable.event));
+		ret = cmd_disable_event(cmd_ctx, *sock);
 		break;
 	}
 	case LTTNG_ENABLE_CHANNEL:
 	{
-		cmd_ctx->lsm->u.channel.chan.attr.extended.ptr =
-				(struct lttng_channel_extended *) &cmd_ctx->lsm->u.channel.extended;
-		ret = cmd_enable_channel(cmd_ctx->session,
-				ALIGNED_CONST_PTR(cmd_ctx->lsm->domain),
-				ALIGNED_CONST_PTR(cmd_ctx->lsm->u.channel.chan),
-				kernel_poll_pipe[1]);
+		ret = cmd_enable_channel(cmd_ctx, *sock, kernel_poll_pipe[1]);
 		break;
 	}
 	case LTTNG_PROCESS_ATTR_TRACKER_ADD_INCLUDE_VALUE:
@@ -1390,162 +1187,32 @@ error_add_context:
 	}
 	case LTTNG_ENABLE_EVENT:
 	{
-		struct lttng_event *ev = NULL;
-		struct lttng_event_exclusion *exclusion = NULL;
-		struct lttng_filter_bytecode *bytecode = NULL;
-		char *filter_expression = NULL;
-
-		/* Handle exclusion events and receive it from the client. */
-		if (cmd_ctx->lsm->u.enable.exclusion_count > 0) {
-			size_t count = cmd_ctx->lsm->u.enable.exclusion_count;
-
-			exclusion = zmalloc(sizeof(struct lttng_event_exclusion) +
-					(count * LTTNG_SYMBOL_NAME_LEN));
-			if (!exclusion) {
-				ret = LTTNG_ERR_EXCLUSION_NOMEM;
-				goto error;
-			}
-
-			DBG("Receiving var len exclusion event list from client ...");
-			exclusion->count = count;
-			ret = lttcomm_recv_unix_sock(*sock, exclusion->names,
-					count * LTTNG_SYMBOL_NAME_LEN);
-			if (ret <= 0) {
-				DBG("Nothing recv() from client var len data... continuing");
-				*sock_error = 1;
-				free(exclusion);
-				ret = LTTNG_ERR_EXCLUSION_INVAL;
-				goto error;
-			}
-		}
-
-		/* Get filter expression from client. */
-		if (cmd_ctx->lsm->u.enable.expression_len > 0) {
-			size_t expression_len =
-				cmd_ctx->lsm->u.enable.expression_len;
-
-			if (expression_len > LTTNG_FILTER_MAX_LEN) {
-				ret = LTTNG_ERR_FILTER_INVAL;
-				free(exclusion);
-				goto error;
-			}
-
-			filter_expression = zmalloc(expression_len);
-			if (!filter_expression) {
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_NOMEM;
-				goto error;
-			}
-
-			/* Receive var. len. data */
-			DBG("Receiving var len filter's expression from client ...");
-			ret = lttcomm_recv_unix_sock(*sock, filter_expression,
-				expression_len);
-			if (ret <= 0) {
-				DBG("Nothing recv() from client var len data... continuing");
-				*sock_error = 1;
-				free(filter_expression);
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_INVAL;
-				goto error;
-			}
-		}
-
-		/* Handle filter and get bytecode from client. */
-		if (cmd_ctx->lsm->u.enable.bytecode_len > 0) {
-			size_t bytecode_len = cmd_ctx->lsm->u.enable.bytecode_len;
-
-			if (bytecode_len > LTTNG_FILTER_MAX_LEN) {
-				ret = LTTNG_ERR_FILTER_INVAL;
-				free(filter_expression);
-				free(exclusion);
-				goto error;
-			}
-
-			bytecode = zmalloc(bytecode_len);
-			if (!bytecode) {
-				free(filter_expression);
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_NOMEM;
-				goto error;
-			}
-
-			/* Receive var. len. data */
-			DBG("Receiving var len filter's bytecode from client ...");
-			ret = lttcomm_recv_unix_sock(*sock, bytecode, bytecode_len);
-			if (ret <= 0) {
-				DBG("Nothing recv() from client var len data... continuing");
-				*sock_error = 1;
-				free(filter_expression);
-				free(bytecode);
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_INVAL;
-				goto error;
-			}
-
-			if ((bytecode->len + sizeof(*bytecode)) != bytecode_len) {
-				free(filter_expression);
-				free(bytecode);
-				free(exclusion);
-				ret = LTTNG_ERR_FILTER_INVAL;
-				goto error;
-			}
-		}
-
-		ev = lttng_event_copy(ALIGNED_CONST_PTR(cmd_ctx->lsm->u.enable.event));
-		if (!ev) {
-			DBG("Failed to copy event: %s",
-					cmd_ctx->lsm->u.enable.event.name);
-			free(filter_expression);
-			free(bytecode);
-			free(exclusion);
-			ret = LTTNG_ERR_NOMEM;
-			goto error;
-		}
-
-
-		if (cmd_ctx->lsm->u.enable.userspace_probe_location_len > 0) {
-			/* Expect a userspace probe description. */
-			ret = receive_userspace_probe(cmd_ctx, *sock, sock_error, ev);
-			if (ret) {
-				free(filter_expression);
-				free(bytecode);
-				free(exclusion);
-				lttng_event_destroy(ev);
-				goto error;
-			}
-		}
-
-		ret = cmd_enable_event(cmd_ctx->session,
-				ALIGNED_CONST_PTR(cmd_ctx->lsm->domain),
-				cmd_ctx->lsm->u.enable.channel_name,
-				ev,
-				filter_expression, bytecode, exclusion,
-				kernel_poll_pipe[1]);
-		lttng_event_destroy(ev);
+		ret = cmd_enable_event(cmd_ctx, *sock, kernel_poll_pipe[1]);
 		break;
 	}
 	case LTTNG_LIST_TRACEPOINTS:
 	{
-		struct lttng_event *events;
-		ssize_t nb_events;
+		enum lttng_error_code ret_code;
+		unsigned int nb_events;
+		struct lttcomm_list_command_header cmd_header = { 0 };
 
 		session_lock_list();
-		nb_events = cmd_list_tracepoints(cmd_ctx->lsm->domain.type, &events);
+		ret_code = cmd_list_tracepoints(cmd_ctx->lsm->domain.type,
+				&payload, &nb_events);
 		session_unlock_list();
-		if (nb_events < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -nb_events;
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
-		/*
-		 * Setup lttng message with payload size set to the event list size in
-		 * bytes and then copy list into the llm payload.
-		 */
-		ret = setup_lttng_msg_no_cmd_header(cmd_ctx, events,
-			sizeof(struct lttng_event) * nb_events);
-		free(events);
+		if (nb_events > UINT32_MAX) {
+			ret = LTTNG_ERR_OVERFLOW;
+			goto error;
+		}
+
+		cmd_header.count = (uint32_t) nb_events;
+		ret = setup_lttng_msg(cmd_ctx, payload.data, payload.size,
+				&cmd_header, sizeof(cmd_header));
 
 		if (ret < 0) {
 			goto setup_error;
@@ -1556,26 +1223,28 @@ error_add_context:
 	}
 	case LTTNG_LIST_TRACEPOINT_FIELDS:
 	{
-		struct lttng_event_field *fields;
-		ssize_t nb_fields;
+		enum lttng_error_code ret_code;
+		unsigned int nb_fields;
+		struct lttcomm_list_command_header cmd_header;
 
 		session_lock_list();
-		nb_fields = cmd_list_tracepoint_fields(cmd_ctx->lsm->domain.type,
-				&fields);
+		ret_code = cmd_list_tracepoint_fields(
+				cmd_ctx->lsm->domain.type, &payload, &nb_fields);
 		session_unlock_list();
-		if (nb_fields < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -nb_fields;
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
-		/*
-		 * Setup lttng message with payload size set to the event list size in
-		 * bytes and then copy list into the llm payload.
-		 */
-		ret = setup_lttng_msg_no_cmd_header(cmd_ctx, fields,
-				sizeof(struct lttng_event_field) * nb_fields);
-		free(fields);
+		if (nb_fields > UINT32_MAX) {
+			ret = LTTNG_ERR_OVERFLOW;
+			goto error;
+		}
+
+		cmd_header.count = nb_fields;
+
+		ret = setup_lttng_msg(cmd_ctx, payload.data, payload.size,
+				&cmd_header, sizeof(cmd_header));
 
 		if (ret < 0) {
 			goto setup_error;
@@ -1586,23 +1255,24 @@ error_add_context:
 	}
 	case LTTNG_LIST_SYSCALLS:
 	{
-		struct lttng_event *events;
-		ssize_t nb_events;
+		enum lttng_error_code ret_code;
+		uint32_t nb_syscalls;
+		struct lttcomm_list_command_header cmd_header = { 0 };
 
-		nb_events = cmd_list_syscalls(&events);
-		if (nb_events < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -nb_events;
+		ret_code = cmd_list_syscalls(&payload, &nb_syscalls);
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
-		/*
-		 * Setup lttng message with payload size set to the event list size in
-		 * bytes and then copy list into the llm payload.
-		 */
-		ret = setup_lttng_msg_no_cmd_header(cmd_ctx, events,
-			sizeof(struct lttng_event) * nb_events);
-		free(events);
+		if (nb_syscalls > UINT32_MAX) {
+			ret = LTTNG_ERR_OVERFLOW;
+			goto error;
+		}
+
+		cmd_header.count = (uint32_t) nb_syscalls;
+		ret = setup_lttng_msg(cmd_ctx, payload.data, payload.size,
+				&cmd_header, sizeof(cmd_header));
 
 		if (ret < 0) {
 			goto setup_error;
@@ -1706,20 +1376,20 @@ error_add_context:
 	}
 	case LTTNG_LIST_CHANNELS:
 	{
-		ssize_t payload_size;
-		struct lttng_channel *channels = NULL;
+		uint32_t nb_channel;
+		enum lttng_error_code ret_code;
+		struct lttcomm_list_command_header cmd_header = { 0 };
 
-		payload_size = cmd_list_channels(cmd_ctx->lsm->domain.type,
-				cmd_ctx->session, &channels);
-		if (payload_size < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -payload_size;
+		ret_code = cmd_list_channels(cmd_ctx->lsm->domain.type,
+				cmd_ctx->session, &payload, &nb_channel);
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
-		ret = setup_lttng_msg_no_cmd_header(cmd_ctx, channels,
-			payload_size);
-		free(channels);
+		cmd_header.count = nb_channel;
+		ret = setup_lttng_msg(cmd_ctx, payload.data, payload.size,
+				&cmd_header, sizeof(cmd_header));
 
 		if (ret < 0) {
 			goto setup_error;
@@ -1730,27 +1400,28 @@ error_add_context:
 	}
 	case LTTNG_LIST_EVENTS:
 	{
-		ssize_t nb_event;
-		struct lttng_event *events = NULL;
-		struct lttcomm_event_command_header cmd_header;
-		size_t total_size;
+		enum lttng_error_code ret_code;
+		unsigned int nb_events;
+		struct lttcomm_list_command_header cmd_header = { 0 };
 
-		memset(&cmd_header, 0, sizeof(cmd_header));
-		/* Extended infos are included at the end of events */
-		nb_event = cmd_list_events(cmd_ctx->lsm->domain.type,
-			cmd_ctx->session, cmd_ctx->lsm->u.list.channel_name,
-			&events, &total_size);
+		ret_code = cmd_list_events(cmd_ctx->lsm->domain.type,
+				cmd_ctx->session,
+				cmd_ctx->lsm->u.list.channel_name, &payload,
+				&nb_events);
 
-		if (nb_event < 0) {
-			/* Return value is a negative lttng_error_code. */
-			ret = -nb_event;
+		if (ret_code != LTTNG_OK) {
+			ret = (int) ret_code;
 			goto error;
 		}
 
-		cmd_header.nb_events = nb_event;
-		ret = setup_lttng_msg(cmd_ctx, events, total_size,
-			&cmd_header, sizeof(cmd_header));
-		free(events);
+		if (nb_events > UINT32_MAX) {
+			ret = LTTNG_ERR_OVERFLOW;
+			goto error;
+		}
+
+		cmd_header.count = (uint32_t) nb_events;
+		ret = setup_lttng_msg(cmd_ctx, payload.data, payload.size,
+				&cmd_header, sizeof(cmd_header));
 
 		if (ret < 0) {
 			goto setup_error;
@@ -2108,13 +1779,13 @@ setup_error:
 	}
 init_setup_error:
 	assert(!rcu_read_ongoing());
+	lttng_dynamic_buffer_reset(&payload);
 	return ret;
 }
 
 static int create_client_sock(void)
 {
 	int ret, client_sock;
-	const mode_t old_umask = umask(0);
 
 	/* Create client tool unix socket */
 	client_sock = lttcomm_create_unix_sock(config.client_unix_sock_path.value);
@@ -2145,7 +1816,6 @@ static int create_client_sock(void)
 	DBG("Created client socket (fd = %i)", client_sock);
 	ret = client_sock;
 end:
-	umask(old_umask);
 	return ret;
 }
 
