@@ -1,18 +1,8 @@
 /*
- * Copyright (C) 2011 - David Goulet <david.goulet@polymtl.ca>
+ * Copyright (C) 2011 EfficiOS Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License, version 2 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #define _LGPL_SOURCE
@@ -29,7 +19,6 @@
 
 #include "../command.h"
 
-static char *opt_event_list;
 static int opt_kernel;
 static char *opt_channel_name;
 static char *opt_session_name;
@@ -116,7 +105,7 @@ const char *print_event_type(const enum lttng_event_type ev_type)
  * enabled is 0 or 1
  * success is 0 or 1
  */
-static int mi_print_event(char *event_name, int enabled, int success)
+static int mi_print_event(const char *event_name, int enabled, int success)
 {
 	int ret;
 
@@ -161,7 +150,7 @@ end:
  *
  *  Disabling event using the lttng API.
  */
-static int disable_events(char *session_name)
+static int disable_events(char *session_name, char *event_list)
 {
 	int ret = CMD_SUCCESS, warn = 0, command_ret = CMD_SUCCESS;
 	int enabled = 1, success = 1;
@@ -250,7 +239,7 @@ static int disable_events(char *session_name)
 		}
 	} else {
 		/* Strip event list */
-		event_name = strtok(opt_event_list, ",");
+		event_name = strtok(event_list, ",");
 		while (event_name != NULL) {
 			DBG("Disabling event %s", event_name);
 
@@ -328,6 +317,8 @@ int cmd_disable_events(int argc, const char **argv)
 	int opt, ret = CMD_SUCCESS, command_ret = CMD_SUCCESS, success = 1;
 	static poptContext pc;
 	char *session_name = NULL;
+	char *event_list = NULL;
+	const char *arg_event_list = NULL;
 	const char *leftover = NULL;
 	int event_type = -1;
 
@@ -394,13 +385,22 @@ int cmd_disable_events(int argc, const char **argv)
 		goto end;
 	}
 
-	opt_event_list = (char*) poptGetArg(pc);
-	if (opt_event_list == NULL && opt_disable_all == 0) {
+	arg_event_list = poptGetArg(pc);
+	if (arg_event_list == NULL && opt_disable_all == 0) {
 		ERR("Missing event name(s).\n");
 		ret = CMD_ERROR;
 		goto end;
 	}
 
+	if (opt_disable_all == 0) {
+		event_list = strdup(arg_event_list);
+		if (event_list == NULL) {
+			PERROR("Failed to copy event name(s)");
+			ret = CMD_ERROR;
+			goto end;
+		}
+	}
+
 	leftover = poptGetArg(pc);
 	if (leftover) {
 		ERR("Unknown argument: %s", leftover);
@@ -443,7 +443,7 @@ int cmd_disable_events(int argc, const char **argv)
 		}
 	}
 
-	command_ret = disable_events(session_name);
+	command_ret = disable_events(session_name, event_list);
 	if (command_ret) {
 		success = 0;
 	}
@@ -477,6 +477,8 @@ end:
 		free(session_name);
 	}
 
+	free(event_list);
+
 	/* Mi clean-up */
 	if (writer && mi_lttng_writer_destroy(writer)) {
 		/* Preserve original error code */
