@@ -1,20 +1,10 @@
 /*
- * Copyright (C) 2011 - Julien Desfossez <julien.desfossez@polymtl.ca>
- *                      Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
- * Copyright (C) 2013 - David Goulet <dgoulet@efficios.com>
+ * Copyright (C) 2011 EfficiOS Inc.
+ * Copyright (C) 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+ * Copyright (C) 2013 David Goulet <dgoulet@efficios.com>
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License, version 2 only, as
- * published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 51
- * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #define _LGPL_SOURCE
@@ -24,14 +14,15 @@
 #include <unistd.h>
 
 #include <common/common.h>
+#include <common/consumer/consumer-timer.h>
+#include <common/consumer/consumer.h>
+#include <common/consumer/metadata-bucket.h>
 #include <common/index/index.h>
 #include <common/kernel-consumer/kernel-consumer.h>
+#include <common/macros.h>
 #include <common/relayd/relayd.h>
 #include <common/ust-consumer/ust-consumer.h>
 #include <common/utils.h>
-#include <common/consumer/consumer.h>
-#include <common/consumer/consumer-timer.h>
-#include <common/consumer/metadata-bucket.h>
 
 #include "consumer-stream.h"
 
@@ -61,6 +52,12 @@ static void consumer_stream_data_unlock_all(struct lttng_consumer_stream *stream
 	pthread_mutex_unlock(&stream->chan->lock);
 }
 
+static void consumer_stream_data_assert_locked_all(struct lttng_consumer_stream *stream)
+{
+	ASSERT_LOCKED(stream->lock);
+	ASSERT_LOCKED(stream->chan->lock);
+}
+
 static void consumer_stream_metadata_lock_all(struct lttng_consumer_stream *stream)
 {
 	consumer_stream_data_lock_all(stream);
@@ -73,6 +70,12 @@ static void consumer_stream_metadata_unlock_all(struct lttng_consumer_stream *st
 	consumer_stream_data_unlock_all(stream);
 }
 
+static void consumer_stream_metadata_assert_locked_all(struct lttng_consumer_stream *stream)
+{
+	ASSERT_LOCKED(stream->metadata_rdv_lock);
+	consumer_stream_data_assert_locked_all(stream);
+}
+
 /* Only used for data streams. */
 static int consumer_stream_update_stats(struct lttng_consumer_stream *stream,
 		const struct stream_subbuffer *subbuf)
@@ -413,6 +416,7 @@ static int consumer_stream_sync_metadata_index(
 		const struct stream_subbuffer *subbuffer,
 		struct lttng_consumer_local_data *ctx)
 {
+	bool missed_metadata_flush;
 	int ret;
 
 	/* Block until all the metadata is sent. */
@@ -425,18 +429,34 @@ static int consumer_stream_sync_metadata_index(
 
 	pthread_mutex_lock(&stream->metadata_timer_lock);
 	stream->waiting_on_metadata = false;
-	if (stream->missed_metadata_flush) {
+	missed_metadata_flush = stream->missed_metadata_flush;
+	if (missed_metadata_flush) {
 		stream->missed_metadata_flush = false;
-		pthread_mutex_unlock(&stream->metadata_timer_lock);
-		(void) stream->read_subbuffer_ops.send_live_beacon(stream);
-	} else {
-		pthread_mutex_unlock(&stream->metadata_timer_lock);
 	}
+	pthread_mutex_unlock(&stream->metadata_timer_lock);
 	if (ret < 0) {
 		goto end;
 	}
 
 	ret = consumer_stream_send_index(stream, subbuffer, ctx);
+	/*
+	 * Send the live inactivity beacon to handle the situation where
+	 * the live timer is prevented from sampling this stream
+	 * because the stream lock was being held while this stream is
+	 * waiting on metadata. This ensures live viewer progress in the
+	 * unlikely scenario where a live timer would be prevented from
+	 * locking a stream lock repeatedly due to a steady flow of
+	 * incoming metadata, for a stream which is mostly inactive.
+	 *
+	 * It is important to send the inactivity beacon packet to
+	 * relayd _after_ sending the index associated with the data
+	 * that was just sent, otherwise this can cause live viewers to
+	 * observe timestamps going backwards between an inactivity
+	 * beacon and a following trace packet.
+	 */
+	if (missed_metadata_flush) {
+		(void) stream->read_subbuffer_ops.send_live_beacon(stream);
+	}
 end:
 	return ret;
 }
@@ -489,13 +509,16 @@ struct lttng_consumer_stream *consumer_stream_create(
 		goto end;
 	}
 
+	rcu_read_lock();
+
 	if (trace_chunk && !lttng_trace_chunk_get(trace_chunk)) {
 		ERR("Failed to acquire trace chunk reference during the creation of a stream");
 		ret = -1;
 		goto error;
 	}
 
-	rcu_read_lock();
+	stream->send_node = (typeof(stream->send_node))
+			CDS_LIST_HEAD_INIT(stream->send_node);
 	stream->chan = channel;
 	stream->key = stream_key;
 	stream->trace_chunk = trace_chunk;
@@ -509,6 +532,8 @@ struct lttng_consumer_stream *consumer_stream_create(
 	stream->index_file = NULL;
 	stream->last_sequence_number = -1ULL;
 	stream->rotate_position = -1ULL;
+	/* Buffer is created with an open packet. */
+	stream->opened_packet_in_current_trace_chunk = true;
 	pthread_mutex_init(&stream->lock, NULL);
 	pthread_mutex_init(&stream->metadata_timer_lock, NULL);
 
@@ -566,12 +591,16 @@ struct lttng_consumer_stream *consumer_stream_create(
 				consumer_stream_metadata_lock_all;
 		stream->read_subbuffer_ops.unlock =
 				consumer_stream_metadata_unlock_all;
+		stream->read_subbuffer_ops.assert_locked =
+				consumer_stream_metadata_assert_locked_all;
 		stream->read_subbuffer_ops.pre_consume_subbuffer =
 				metadata_stream_check_version;
 	} else {
 		stream->read_subbuffer_ops.lock = consumer_stream_data_lock_all;
 		stream->read_subbuffer_ops.unlock =
 				consumer_stream_data_unlock_all;
+		stream->read_subbuffer_ops.assert_locked =
+				consumer_stream_data_assert_locked_all;
 		stream->read_subbuffer_ops.pre_consume_subbuffer =
 				consumer_stream_update_stats;
 		if (channel->is_live) {
@@ -862,6 +891,8 @@ void consumer_stream_destroy(struct lttng_consumer_stream *stream,
 {
 	assert(stream);
 
+	cds_list_del_init(&stream->send_node);
+
 	/* Stream is in monitor mode. */
 	if (stream->monitor) {
 		struct lttng_consumer_channel *free_chan = NULL;
@@ -894,6 +925,7 @@ void consumer_stream_destroy(struct lttng_consumer_stream *stream,
 			 * If the stream is not visible globally, this needs to be done
 			 * outside of the consumer data lock section.
 			 */
+			destroy_close_stream(stream);
 			free_chan = unref_channel(stream);
 		}
 
@@ -994,7 +1026,7 @@ int consumer_stream_create_output_files(struct lttng_consumer_stream *stream,
 
 	DBG("Opening stream output file \"%s\"", stream_path);
 	chunk_status = lttng_trace_chunk_open_file(stream->trace_chunk, stream_path,
-			flags, mode, &stream->out_fd);
+			flags, mode, &stream->out_fd, false);
         if (chunk_status != LTTNG_TRACE_CHUNK_STATUS_OK) {
 		ERR("Failed to open stream file \"%s\"", stream->name);
 		ret = -1;
@@ -1005,15 +1037,15 @@ int consumer_stream_create_output_files(struct lttng_consumer_stream *stream,
 		if (stream->index_file) {
 			lttng_index_file_put(stream->index_file);
 		}
-                stream->index_file = lttng_index_file_create_from_trace_chunk(
+		chunk_status = lttng_index_file_create_from_trace_chunk(
 				stream->trace_chunk,
 				stream->chan->pathname,
 				stream->name,
 				stream->chan->tracefile_size,
 				stream->tracefile_count_current,
 				CTF_INDEX_MAJOR, CTF_INDEX_MINOR,
-				false);
-		if (!stream->index_file) {
+				false, &stream->index_file);
+		if (chunk_status != LTTNG_TRACE_CHUNK_STATUS_OK) {
 			ret = -1;
 			goto end;
 		}
