@@ -6,6 +6,7 @@
  */
 
 #include <common/dynamic-buffer.h>
+#include <common/buffer-view.h>
 #include <common/utils.h>
 #include <assert.h>
 
@@ -83,6 +84,23 @@ end:
 	return ret;
 }
 
+LTTNG_HIDDEN
+int lttng_dynamic_buffer_append_view(struct lttng_dynamic_buffer *buffer,
+		const struct lttng_buffer_view *src)
+{
+	int ret;
+
+	if (!buffer || !src) {
+		ret = -1;
+		goto end;
+	}
+
+	ret = lttng_dynamic_buffer_append(buffer, src->data,
+			src->size);
+end:
+	return ret;
+}
+
 LTTNG_HIDDEN
 int lttng_dynamic_buffer_set_size(struct lttng_dynamic_buffer *buffer,
 		size_t new_size)
