@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2011 David Goulet <david.goulet@polymtl.ca>
+ * Copyright (C) 2011 EfficiOS Inc.
  * Copyright (C) 2011 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
- * SPDX-License-Identifier: GPL-2.0-only
+ * SPDX-License-Identifier: LGPL-2.1-only
  *
  */
 
@@ -210,6 +210,9 @@ ssize_t lttcomm_recv_unix_sock(int sock, void *buf, size_t len)
  * Receive data of size len in put that data into the buf param. Using recvmsg
  * API. Only use with sockets set in non-blocking mode.
  *
+ * NOTE: EPIPE errors are NOT reported. This call expects the socket to be in a
+ * poll set. The poll loop will handle the EPIPE original cause.
+ *
  * Return the size of received data.
  */
 LTTNG_HIDDEN
@@ -233,17 +236,24 @@ retry:
 			goto retry;
 		} else {
 			/*
-			 * Only warn about EPIPE when quiet mode is
-			 * deactivated.
-			 * We consider EPIPE as expected.
+			 * We consider EPIPE and EAGAIN/EWOULDBLOCK as expected.
 			 */
-			if (errno != EPIPE || !lttng_opt_quiet) {
-				PERROR("recvmsg");
+			if (errno == EAGAIN || errno == EWOULDBLOCK ||
+					errno == EPIPE) {
+				/*
+				 * Nothing was recv.
+				 */
+				ret = 0;
+				goto end;
 			}
+
+			/* Unexpected error */
+			PERROR("recvmsg");
+			ret = -1;
 			goto end;
 		}
 	}
-	ret = len;
+
 end:
 	return ret;
 }
@@ -298,6 +308,9 @@ end:
  * of the function is that this one does not retry to send on partial sends,
  * except if the interruption was caused by a signal (EINTR).
  *
+ * NOTE: EPIPE errors are NOT reported. This call expects the socket to be in a
+ * poll set. The poll loop will handle the EPIPE original cause.
+ *
  * Return the size of sent data.
  */
 LTTNG_HIDDEN
@@ -321,17 +334,24 @@ retry:
 			goto retry;
 		} else {
 			/*
-			 * Only warn about EPIPE when quiet mode is
-			 * deactivated.
-			 * We consider EPIPE as expected.
+			 * We consider EPIPE and EAGAIN/EWOULDBLOCK as expected.
 			 */
-			if (errno != EPIPE || !lttng_opt_quiet) {
-				PERROR("sendmsg");
+			if (errno == EAGAIN || errno == EWOULDBLOCK ||
+					errno == EPIPE) {
+				/*
+				 * This can happen in non blocking mode.
+				 * Nothing was sent.
+				 */
+				ret = 0;
+				goto end;
 			}
+
+			/* Unexpected error */
+			PERROR("sendmsg");
+			ret = -1;
 			goto end;
 		}
 	}
-	ret = len;
 end:
 	return ret;
 }
