@@ -54,6 +54,10 @@ int receive_message(struct lttng_notification_channel *channel)
 		goto error;
 	}
 
+	if (msg.size == 0) {
+		goto skip_payload;
+	}
+
 	/* Reserve space for the payload. */
 	ret = lttng_dynamic_buffer_set_size(&channel->reception_buffer,
 			channel->reception_buffer.size + msg.size);
@@ -68,6 +72,8 @@ int receive_message(struct lttng_notification_channel *channel)
 		ret = -1;
 		goto error;
 	}
+
+skip_payload:
 	ret = 0;
 end:
 	return ret;
@@ -151,9 +157,14 @@ struct lttng_notification_channel *lttng_notification_channel_create(
 	}
 
 	if (is_root || is_in_tracing_group) {
-		lttng_ctl_copy_string(sock_path,
+		ret = lttng_strncpy(sock_path,
 				DEFAULT_GLOBAL_NOTIFICATION_CHANNEL_UNIX_SOCK,
 				LTTNG_PATH_MAX);
+		if (ret) {
+			ret = -LTTNG_ERR_INVALID;
+			goto error;
+		}
+
 		ret = lttcomm_connect_unix_sock(sock_path);
 		if (ret >= 0) {
 			fd = ret;
