@@ -1,19 +1,8 @@
 /*
- * Copyright (c)  2011 David Goulet <david.goulet@polymtl.ca>
+ * Copyright (C) 2011 EfficiOS Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * as published by the Free Software Foundation; only version 2
- * of the License.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <assert.h>
@@ -30,6 +19,7 @@
 #include <common/defaults.h>
 #include <bin/lttng-sessiond/trace-ust.h>
 #include <bin/lttng-sessiond/ust-app.h>
+#include <bin/lttng-sessiond/notification-thread.h>
 
 #include <tap/tap.h>
 
@@ -46,21 +36,12 @@ int lttng_opt_quiet = 1;
 int lttng_opt_verbose;
 int lttng_opt_mi;
 
-int ust_consumerd32_fd;
-int ust_consumerd64_fd;
-
-/* Global variable required by sessiond objects being linked-in */
-struct lttng_ht *agent_apps_ht_by_sock;
-
 static const char alphanum[] =
 	"0123456789"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 	"abcdefghijklmnopqrstuvwxyz";
 static char random_string[RANDOM_STRING_LEN];
 
-static struct ltt_ust_session *usess;
-static struct lttng_domain dom;
-
 /*
  * Return random string of 10 characters.
  * Not thread-safe.
@@ -80,9 +61,9 @@ static char *get_random_string(void)
 
 static void test_create_one_ust_session(void)
 {
-	dom.type = LTTNG_DOMAIN_UST;
+	struct ltt_ust_session *usess =
+		trace_ust_create_session(42);
 
-	usess = trace_ust_create_session(42);
 	ok(usess != NULL, "Create UST session");
 
 	if (!usess) {
@@ -98,22 +79,26 @@ static void test_create_one_ust_session(void)
 	   "Validate UST session");
 
 	trace_ust_destroy_session(usess);
+	trace_ust_free_session(usess);
 }
 
 static void test_create_ust_channel(void)
 {
 	struct ltt_ust_channel *uchan;
 	struct lttng_channel attr;
+	struct lttng_channel_extended extended;
 
 	memset(&attr, 0, sizeof(attr));
+	memset(&extended, 0, sizeof(extended));
+	attr.attr.extended.ptr = &extended;
 
 	ok(lttng_strncpy(attr.name, "channel0", sizeof(attr.name)) == 0,
 		"Validate channel name length");
 	uchan = trace_ust_create_channel(&attr, LTTNG_DOMAIN_UST);
 	ok(uchan != NULL, "Create UST channel");
 
-	if (!usess) {
-		skip(1, "UST session is null");
+	if (!uchan) {
+		skip(1, "UST channel is null");
 		return;
 	}
 
@@ -132,6 +117,7 @@ static void test_create_ust_event(void)
 {
 	struct ltt_ust_event *event;
 	struct lttng_event ev;
+	enum lttng_error_code ret;
 
 	memset(&ev, 0, sizeof(ev));
 	ok(lttng_strncpy(ev.name, get_random_string(),
@@ -140,9 +126,9 @@ static void test_create_ust_event(void)
 	ev.type = LTTNG_EVENT_TRACEPOINT;
 	ev.loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
 
-	event = trace_ust_create_event(&ev, NULL, NULL, NULL, false);
+	ret = trace_ust_create_event(&ev, NULL, NULL, NULL, false, &event);
 
-	ok(event != NULL, "Create UST event");
+	ok(ret == LTTNG_OK, "Create UST event");
 
 	if (!event) {
 		skip(1, "UST event is null");
@@ -160,11 +146,13 @@ static void test_create_ust_event(void)
 
 static void test_create_ust_event_exclusion(void)
 {
+	enum lttng_error_code ret;
 	struct ltt_ust_event *event;
 	struct lttng_event ev;
 	char *name;
 	char *random_name;
-	struct lttng_event_exclusion *exclusion;
+	struct lttng_event_exclusion *exclusion = NULL;
+	struct lttng_event_exclusion *exclusion_copy = NULL;
 	const int exclusion_count = 2;
 
 	memset(&ev, 0, sizeof(ev));
@@ -190,14 +178,14 @@ static void test_create_ust_event_exclusion(void)
 	exclusion->count = exclusion_count;
 	random_name = get_random_string();
 	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 0), random_name,
-		LTTNG_SYMBOL_NAME_LEN);
+		LTTNG_SYMBOL_NAME_LEN - 1);
 	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 1), random_name,
-		LTTNG_SYMBOL_NAME_LEN);
+		LTTNG_SYMBOL_NAME_LEN - 1);
 
-	event = trace_ust_create_event(&ev, NULL, NULL, exclusion, false);
+	ret = trace_ust_create_event(&ev, NULL, NULL, exclusion, false, &event);
 	exclusion = NULL;
 
-	ok(!event, "Create UST event with identical exclusion names fails");
+	ok(ret != LTTNG_OK, "Create UST event with identical exclusion names fails");
 
 	exclusion = zmalloc(sizeof(*exclusion) +
 		LTTNG_SYMBOL_NAME_LEN * exclusion_count);
@@ -207,14 +195,34 @@ static void test_create_ust_event_exclusion(void)
 		goto end;
 	}
 
+	exclusion_copy = zmalloc(sizeof(*exclusion) +
+		LTTNG_SYMBOL_NAME_LEN * exclusion_count);
+	if (!exclusion_copy) {
+		skip(2, "zmalloc failed");
+		goto end;
+	}
+
+	/*
+	 * We are giving ownership of the exclusion struct to the
+	 * trace_ust_create_event() function. Make a copy of the exclusion struct
+	 * so we can compare it later.
+	 */
+
 	exclusion->count = exclusion_count;
 	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 0),
-		get_random_string(), LTTNG_SYMBOL_NAME_LEN);
+		get_random_string(), LTTNG_SYMBOL_NAME_LEN - 1);
 	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 1),
-		get_random_string(), LTTNG_SYMBOL_NAME_LEN);
+		get_random_string(), LTTNG_SYMBOL_NAME_LEN - 1);
 
-	event = trace_ust_create_event(&ev, NULL, NULL, exclusion, false);
-	ok(event != NULL, "Create UST event with different exclusion names");
+	exclusion_copy->count = exclusion_count;
+	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion_copy, 0),
+		LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 0), LTTNG_SYMBOL_NAME_LEN);
+	strncpy(LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion_copy, 1),
+		LTTNG_EVENT_EXCLUSION_NAME_AT(exclusion, 1), LTTNG_SYMBOL_NAME_LEN);
+
+	ret = trace_ust_create_event(&ev, NULL, NULL, exclusion, false, &event);
+	exclusion = NULL;
+	ok(ret == LTTNG_OK, "Create UST event with different exclusion names");
 
 	if (!event) {
 		skip(1, "UST event with exclusion is null");
@@ -222,17 +230,19 @@ static void test_create_ust_event_exclusion(void)
 	}
 
 	ok(event->enabled == 0 &&
-	   event->attr.instrumentation == LTTNG_UST_TRACEPOINT &&
-	   strcmp(event->attr.name, ev.name) == 0 &&
-	   event->exclusion != NULL &&
-	   event->exclusion->count == exclusion_count &&
-	   !memcmp(event->exclusion->names, exclusion->names,
-	   	LTTNG_SYMBOL_NAME_LEN * exclusion_count) &&
-	   event->attr.name[LTTNG_UST_SYM_NAME_LEN - 1] == '\0',
-	   "Validate UST event and exclusion");
+		event->attr.instrumentation == LTTNG_UST_TRACEPOINT &&
+		strcmp(event->attr.name, ev.name) == 0 &&
+		event->exclusion != NULL &&
+		event->exclusion->count == exclusion_count &&
+		!memcmp(event->exclusion->names, exclusion_copy->names,
+			LTTNG_SYMBOL_NAME_LEN * exclusion_count) &&
+		event->attr.name[LTTNG_UST_SYM_NAME_LEN - 1] == '\0',
+		"Validate UST event and exclusion");
 
 	trace_ust_destroy_event(event);
 end:
+	free(exclusion);
+	free(exclusion_copy);
 	return;
 }
 
@@ -247,8 +257,12 @@ static void test_create_ust_context(void)
 	uctx = trace_ust_create_context(&ectx);
 	ok(uctx != NULL, "Create UST context");
 
-	ok((int) uctx->ctx.ctx == LTTNG_UST_CONTEXT_VTID,
-	   "Validate UST context");
+	if (uctx) {
+		ok((int) uctx->ctx.ctx == LTTNG_UST_CONTEXT_VTID,
+		   "Validate UST context");
+	} else {
+		skip(1, "Skipping UST context validation as creation failed");
+	}
 	free(uctx);
 }
 
