@@ -16,6 +16,11 @@ BABELTRACE_BIN="babeltrace"
 OUTPUT_DEST=/dev/null
 ERROR_OUTPUT_DEST=/dev/null
 
+# To match 20201127-175802
+date_time_pattern="[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]-[0-9][0-9][0-9][0-9][0-9][0-9]"
+# The size of a long on this system
+system_long_bit_size=$(getconf LONG_BIT)
+
 # Minimal kernel version supported for session daemon tests
 KERNEL_MAJOR_VERSION=2
 KERNEL_MINOR_VERSION=6
@@ -125,6 +130,105 @@ function randstring()
 	echo
 }
 
+# Return a space-separated string of online CPU IDs, based on
+# /sys/devices/system/cpu/online, or from 0 to nproc - 1 otherwise.
+function get_online_cpus()
+{
+	local cpus=()
+	local range_re
+	if [ -f /sys/devices/system/cpu/online ]; then
+		range_re='([0-9]+)-([0-9]+)'
+		while read -r range ; do
+			if [[ "${range}" =~ ${range_re} ]] ; then
+				mapfile -t -O "${#cpus[*]}" cpus <<< $(seq "${BASH_REMATCH[1]}" "${BASH_REMATCH[2]}")
+			else
+				cpus+=("${range}")
+			fi
+		done < <(tr ',' $'\n' < /sys/devices/system/cpu/online)
+	else
+		read -r -a cpus <<< $(seq 0 $(( $(conf_proc_count) - 1 )) )
+	fi
+	echo "${cpus[*]}"
+}
+
+# Helpers for get_possible_cpus.
+function get_possible_cpus_count_from_sysfs_possible_mask()
+{
+	local max_possible_cpu_id
+
+	# The Awk script extracts the highest CPU id from the possible CPU
+	# mask. Assuming a numerical order, a field separator '-' and a record
+	# separator ','. The last value parsed is the highest id.
+	if [ -f /sys/devices/system/cpu/possible ]; then
+		max_possible_cpu_id=$(awk -F '-' 'BEGIN { RS = ","} { last = $NF } END { printf("%d\n", last) }' \
+				      /sys/devices/system/cpu/possible)
+		echo "$((max_possible_cpu_id+1))"
+	else
+		echo "0"
+	fi
+}
+
+# This is a fallback if the possible CPU mask is not available. This will not
+# take into account unplugged CPUs.
+function get_max_cpus_count_from_sysfs_cpu_directories()
+{
+	local max_possible_cpu_id=0
+	local current_cpu_id
+
+	for i in /sys/devices/system/cpu/cpu[0-9]*; do
+		current_cpu_id="${i#/sys/devices/system/cpu/cpu}"
+		if [ "$current_cpu_id" -gt "$max_possible_cpu_id" ]; then
+			max_possible_cpu_id="$current_cpu_id"
+		fi
+	done
+
+	echo "$((max_possible_cpu_id+1))"
+}
+
+# Return the number of possible CPUs.
+function get_possible_cpus_count()
+{
+	local possible_cpus_count
+	possible_cpus_count=$(get_possible_cpus_count_from_sysfs_possible_mask)
+
+	if [ "$possible_cpus_count" -eq "0" ]; then
+		local configured_cpus_count
+		configured_cpus_count=$(getconf _NPROCESSORS_CONF)
+		possible_cpus_count=$(get_max_cpus_count_from_sysfs_cpu_directories)
+		possible_cpus_count=$((configured_cpus_count > possible_cpus_count \
+							     ? configured_cpus_count \
+							     : possible_cpus_count))
+	fi
+
+	echo "$possible_cpus_count"
+}
+
+# Return the list of exposed CPU.
+#
+# NOTE! Use it like so:
+#
+# IFS=" " read -r -a VARIABLE <<< "$(get_exposed_cpus_list)"
+function get_exposed_cpus_list()
+{
+	local list=()
+
+	for i in /sys/devices/system/cpu/cpu[0-9]*; do
+		list+=("${i#/sys/devices/system/cpu/cpu}")
+	done
+
+	echo "${list[@]}"
+}
+
+# Return any available CPU found. Do not make assumption about the returned
+# value, e.g. that it could be 0.
+function get_any_available_cpu()
+{
+	for cpu in $(get_online_cpus); do
+		echo "${cpu}"
+		break;
+	done
+}
+
 # Return the number of _configured_ CPUs.
 function conf_proc_count()
 {
@@ -154,6 +258,20 @@ function validate_lttng_modules_present ()
 	BAIL_OUT "LTTng modules not detected."
 }
 
+# Run the lttng binary.
+#
+# The first two arguments are stdout and stderr redirect paths, respectively.
+# The rest of the arguments are forwarded to the lttng binary
+function _run_lttng_cmd
+{
+	local stdout_dest="$1"
+	local stderr_dest="$2"
+	shift 2
+
+	diag "$TESTDIR/../src/bin/lttng/$LTTNG_BIN $*"
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN "$@" 1> "$stdout_dest" 2> "$stderr_dest"
+}
+
 function enable_kernel_lttng_event
 {
 	local withtap="$1"
@@ -404,7 +522,7 @@ function start_lttng_relayd_opt()
 	if [ -z $(pgrep $RELAYD_MATCH) ]; then
 		# shellcheck disable=SC2086
 		$DIR/../src/bin/lttng-relayd/$RELAYD_BIN $process_mode $opt 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
-		#$DIR/../src/bin/lttng-relayd/$RELAYD_BIN $opt -vvv >>/tmp/relayd.log 2>&1 &
+		#$DIR/../src/bin/lttng-relayd/$RELAYD_BIN $process_mode $opt -vvv >>/tmp/relayd.log 2>&1 &
 		if [ $? -eq 1 ]; then
 			if [ $withtap -eq "1" ]; then
 				fail "Start lttng-relayd (process mode: $process_mode opt: $opt)"
@@ -664,7 +782,7 @@ function stop_lttng_sessiond_opt()
 
 			if [ -n "$modules" ]; then
 				diag "Unloading all LTTng modules"
-				modprobe -r "$modules"
+				modprobe --remove "$modules"
 			fi
 		fi
 	fi
@@ -1106,37 +1224,80 @@ function enable_jul_lttng_event_loglevel()
 
 function enable_log4j_lttng_event()
 {
-	sess_name=$1
-	event_name="$2"
-	channel_name=$3
+	local sess_name=$1
+	local event_name=$2
+	local channel_name=$3
 
-	if [ -z $channel_name ]; then
-		# default channel if none specified
-		chan=""
-	else
-		chan="-c $channel_name"
+	local chan_opt=()
+
+	# default channel if none specified
+	if [ -n "$channel_name" ]; then
+		chan_opt=("-c" "$channel_name")
 	fi
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-event "$event_name" $chan -s $sess_name -l 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
-	ok $? "Enable LOG4J event $event_name for session $sess_name"
+	_run_lttng_cmd "$OUTPUT_DEST" "$ERROR_OUTPUT_DEST" \
+		enable-event "$event_name" "${chan_opt[@]}" -s "$sess_name" --log4j
+	ok $? "Enable LOG4J event '$event_name' for session '$sess_name'"
+}
+
+function enable_log4j_lttng_event_filter()
+{
+	local sess_name=$1
+	local event_name=$2
+	local filter=$3
+
+	_run_lttng_cmd "$OUTPUT_DEST" "$ERROR_OUTPUT_DEST" \
+		enable-event "$event_name" -s "$sess_name" --log4j --filter "$filter"
+	ok $? "Enable LOG4J event '$event_name' with filter '$filter' for session '$sess_name'"
+}
+
+function enable_log4j_lttng_event_filter_loglevel_only()
+{
+	local sess_name=$1
+	local event_name=$2
+	local filter=$3
+	local loglevel=$4
+
+	_run_lttng_cmd "$OUTPUT_DEST" "$ERROR_OUTPUT_DEST" \
+		enable-event --loglevel-only "$loglevel" "$event_name" -s "$sess_name" -l --filter "$filter"
+	ok $? "Enable LOG4J event '$event_name' with filter '$filter' and loglevel-only '$loglevel' for session '$sess_name'"
 }
 
 function enable_log4j_lttng_event_loglevel()
 {
 	local sess_name=$1
-	local event_name="$2"
+	local event_name=$2
 	local loglevel=$3
 	local channel_name=$4
 
-	if [ -z $channel_name ]; then
-		# default channel if none specified
-		chan=""
-	else
-		chan="-c $channel_name"
+
+	# default channel if none specified
+	if [ -n "$channel_name" ]; then
+		chan_opt=("-c" "$channel_name")
 	fi
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN enable-event --loglevel $loglevel "$event_name" $chan -s $sess_name -l 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
-	ok $? "Enable LOG4J event $event_name for session $sess_name with loglevel $loglevel"
+	_run_lttng_cmd "$OUTPUT_DEST" "$ERROR_OUTPUT_DEST" \
+		enable-event --loglevel "$loglevel" "$event_name" "${chan_opt[@]}" -s "$sess_name" --log4j
+	ok $? "Enable LOG4J event '$event_name' for session '$sess_name' with loglevel '$loglevel'"
+}
+
+function enable_log4j_lttng_event_loglevel_only()
+{
+	local sess_name=$1
+	local event_name=$2
+	local loglevel=$3
+	local channel_name=$4
+
+	local chan_opt=()
+
+	# default channel if none specified
+	if [ -n "$channel_name" ]; then
+		chan_opt=("-c" "$channel_name")
+	fi
+
+	_run_lttng_cmd "$OUTPUT_DEST" "$ERROR_OUTPUT_DEST" \
+		enable-event --loglevel-only "$loglevel" "$event_name" "${chan_opt[@]}" -s "$sess_name" --log4j
+	ok $? "Enable LOG4J event '$event_name' for session '$sess_name' with loglevel-only '$loglevel'"
 }
 
 function enable_python_lttng_event()
@@ -1243,8 +1404,9 @@ function disable_log4j_lttng_event ()
 	local sess_name="$1"
 	local event_name="$2"
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN disable-event "$event_name" -s $sess_name -l >/dev/null 2>&1
-	ok $? "Disable LOG4J event $event_name for session $sess_name"
+	_run_lttng_cmd "$OUTPUT_DEST" "$ERROR_OUTPUT_DEST" \
+		disable-event "$event_name" -s "$sess_name" --log4j
+	ok $? "Disable LOG4J event '$event_name' for session '$sess_name'"
 }
 
 function disable_python_lttng_event ()
@@ -1427,8 +1589,9 @@ function lttng_snapshot_del_output_fail ()
 function lttng_snapshot_record ()
 {
 	local sess_name=$1
+	local trace_path=$2
 
-	$TESTDIR/../src/bin/lttng/$LTTNG_BIN snapshot record -s $sess_name $trace_path 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
+	$TESTDIR/../src/bin/lttng/$LTTNG_BIN snapshot record -s "$sess_name" "$trace_path" 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ok $? "Snapshot recorded"
 }
 
@@ -1647,6 +1810,14 @@ function wait_live_viewer_connect ()
 	pass "Waiting for live viewers on url: $url"
 }
 
+function bail_out_if_no_babeltrace()
+{
+	which "$BABELTRACE_BIN" >/dev/null
+	if [ $? -ne 0 ]; then
+		LTTNG_BAIL_OUT "\"$BABELTRACE_BIN\" binary not found. Skipping tests"
+	fi
+}
+
 function validate_metadata_event ()
 {
 	local event_name=$1
@@ -1977,7 +2148,7 @@ function rotate_session_fail ()
 
 function destructive_tests_enabled ()
 {
-	if [ ${LTTNG_ENABLE_DESTRUCTIVE_TESTS} = "will-break-my-system" ]; then
+	if [ "$LTTNG_ENABLE_DESTRUCTIVE_TESTS" = "will-break-my-system" ]; then
 		return 0
 	else
 		return 1
@@ -2066,3 +2237,223 @@ function lttng_clear_all ()
 	$TESTDIR/../src/bin/lttng/$LTTNG_BIN clear --all 1> $OUTPUT_DEST 2> $ERROR_OUTPUT_DEST
 	ok $? "Clear all lttng sessions"
 }
+
+function validate_path_pattern ()
+{
+	local message=$1
+	local pattern=$2
+	# Base path is only used in error case and is used to list the content
+	# of the base path.
+	local base_path=$3
+
+
+	[ -f $pattern ]
+	ret=$?
+	ok $ret "$message"
+
+	if [ "$ret" -ne "0" ]; then
+		diag "Path pattern expected: $pattern"
+		# List the tracepath for more info. We use find as a recursive
+		# directory lister.
+		diag "The base path content:"
+		find "$base_path" -print
+	fi
+}
+
+function validate_trace_path_ust_uid ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local uid=$UID
+	local pattern="$trace_path/$session_name-$date_time_pattern/ust/uid/$uid/${system_long_bit_size}-bit/metadata"
+
+	validate_path_pattern "UST per-uid trace path is valid" "$pattern" "$trace_path"
+}
+
+function validate_trace_path_ust_uid_network ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local base_path=$3
+	local uid=$UID
+	local hostname=$HOSTNAME
+	local pattern
+	local ret
+
+	# If the session was given a network base path (e.g
+	# 127.0.0.1/my/custom/path on creation, there is no session name
+	# component to the path on the relayd side. Caller can simply not pass a
+	# session name for this scenario.
+	if [ -n "$session_name" ]; then
+		session_name="$session_name-$date_time_pattern"
+		if [ -n "$base_path" ]; then
+			fail "Session name and base path are mutually exclusive"
+			return
+		fi
+	fi
+
+	pattern="$trace_path/$hostname/$base_path/$session_name/ust/uid/$uid/${system_long_bit_size}-bit/metadata"
+
+	validate_path_pattern "UST per-uid network trace path is valid" "$pattern" "$trace_path"
+}
+
+function validate_trace_path_ust_uid_snapshot_network ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local snapshot_name=$3
+	local snapshot_number=$4
+	local base_path=$5
+	local hostname=$HOSTNAME
+	local uid=$UID
+	local pattern
+	local ret
+
+	# If the session/output was given a network base path (e.g
+	# 127.0.0.1/my/custom/path on creation, there is no session name
+	# component to the path on the relayd side. Caller can simply not pass a
+	# session name for this scenario.
+	if [ -n "$session_name" ]; then
+		session_name="$session_name-$date_time_pattern"
+		if [ -n "$base_path" ]; then
+			fail "Session name and base path are mutually exclusive"
+			return
+		fi
+	fi
+
+	pattern="$trace_path/$hostname/$base_path/$session_name/$snapshot_name-$date_time_pattern-$snapshot_number/ust/uid/$uid/${system_long_bit_size}-bit/metadata"
+
+	validate_path_pattern "UST per-uid network snapshot trace path is valid" "$pattern" "$trace_path"
+}
+
+function validate_trace_path_ust_uid_snapshot ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local snapshot_name=$3
+	local snapshot_number=$4
+	local base_path=$5
+	local uid=$UID
+	local pattern
+	local ret
+
+	# If the session/output was given a network base path (e.g
+	# 127.0.0.1/my/custom/path) on creation, there is no session name
+	# component to the path on the relayd side. Caller can simply not pass a
+	# session name for this scenario.
+	if [ -n "$session_name" ]; then
+		session_name="$session_name-$date_time_pattern"
+		if [ -n "$base_path" ]; then
+			fail "Session name and base path are mutually exclusive"
+			return
+		fi
+	fi
+
+	pattern="$trace_path/$base_path/$session_name/$snapshot_name-$date_time_pattern-$snapshot_number/ust/uid/$uid/${system_long_bit_size}-bit/metadata"
+
+	validate_path_pattern "UST per-uid snapshot trace path is valid" "$pattern" "$trace_path"
+}
+
+function validate_trace_path_ust_pid ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local app_string=$3
+	local pid=$4
+	local pattern
+	local ret
+
+	# If the session was given a trace path on creation, there is no session
+	# name component to the path. Caller can simply not pass a session name
+	# for this scenario.
+	if [ -n "$session_name" ]; then
+		session_name="$session_name-$date_time_pattern"
+	fi
+
+	pattern="$trace_path/$session_name/ust/pid/$pid/$app_string-*-$date_time_pattern/metadata"
+
+	validate_path_pattern "UST per-pid trace path is valid" "$pattern" "$trace_path"
+}
+
+function validate_trace_path_kernel ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local pattern
+
+	# If the session was given a trace path on creation, there is no session
+	# name component to the path. Caller can simply not pass a session name
+	# for this scenario.
+	if [ -n "$session_name" ]; then
+		session_name="$session_name-$date_time_pattern"
+	fi
+
+	pattern="$trace_path/$session_name/kernel/metadata"
+
+	validate_path_pattern "Kernel trace path is valid" "$pattern" "$trace_path"
+}
+
+function validate_trace_path_kernel_network ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local hostname=$HOSTNAME
+	local pattern="$trace_path/$hostname/$session_name-$date_time_pattern/kernel/metadata"
+
+	validate_path_pattern "Kernel network trace path is valid" "$pattern" "$trace_path"
+}
+
+function validate_trace_path_kernel_snapshot ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local snapshot_name=$3
+	local snapshot_number=$4
+	local base_path=$5
+	local pattern
+	local ret
+
+	# If the session/output was given a network base path (e.g
+	# 127.0.0.1/my/custom/path on creation, there is no session name
+	# component to the path on the relayd side. Caller can simply not pass a
+	# session name for this scenario.
+	if [ -n "$session_name" ]; then
+		session_name="$session_name-$date_time_pattern"
+		if [ -n "$base_path" ]; then
+			fail "Session name and base path are mutually exclusive"
+			return
+		fi
+	fi
+
+	pattern="$trace_path/$base_path/$session_name/$snapshot_name-$date_time_pattern-$snapshot_number/kernel/metadata"
+
+	validate_path_pattern "Kernel snapshot trace path is valid" "$pattern" "$trace_path"
+}
+
+function validate_trace_path_kernel_snapshot_network ()
+{
+	local trace_path=$1
+	local session_name=$2
+	local snapshot_name=$3
+	local snapshot_number=$4
+	local base_path=$5
+	local hostname=$HOSTNAME
+	local pattern
+	local ret
+
+	# If the session/output was given a network base path (e.g
+	# 127.0.0.1/my/custom/path on creation, there is no session name
+	# component to the path on the relayd side. Caller can simply not pass a
+	# session name for this scenario.
+	if [ -n "$session_name" ]; then
+		session_name="$session_name-$date_time_pattern"
+		if [ -n "$base_path" ]; then
+			fail "Session name and base path are mutually exclusive"
+			return
+		fi
+	fi
+
+	pattern="$trace_path/$hostname/$base_path/$session_name/$snapshot_name-$date_time_pattern-$snapshot_number/kernel/metadata"
+
+	validate_path_pattern "Kernel network snapshot trace path is valid" "$pattern" "$trace_path"
+}
